"""
Functions that deal with the segment registers.

If your processor doesn't use segment registers, then these functions are of no
use for you. However, you should define two virtual segment registers - CS and
DS (for code segment and data segment) and specify their internal numbers in the
LPH structure (processor_t::reg_code_sreg and processor_t::reg_data_sreg)."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_segregs
else:
    import _ida_segregs

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_segregs.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

import ida_range
R_es = _ida_segregs.R_es

R_cs = _ida_segregs.R_cs

R_ss = _ida_segregs.R_ss

R_ds = _ida_segregs.R_ds

R_fs = _ida_segregs.R_fs

R_gs = _ida_segregs.R_gs

class sreg_range_t(ida_range.range_t):
    r"""
    Proxy of C++ sreg_range_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    val = property(_ida_segregs.sreg_range_t_val_get, _ida_segregs.sreg_range_t_val_set, doc=r"""val""")
    r"""
    segment register value
    """
    tag = property(_ida_segregs.sreg_range_t_tag_get, _ida_segregs.sreg_range_t_tag_set, doc=r"""tag""")
    r"""
    Segment register range tags
    """

    def __init__(self, *args):
        r"""
        __init__(self) -> sreg_range_t
        """
        _ida_segregs.sreg_range_t_swiginit(self, _ida_segregs.new_sreg_range_t(*args))
    __swig_destroy__ = _ida_segregs.delete_sreg_range_t

# Register sreg_range_t in _ida_segregs:
_ida_segregs.sreg_range_t_swigregister(sreg_range_t)
SR_inherit = _ida_segregs.SR_inherit
r"""
the value is inherited from the previous range
"""

SR_user = _ida_segregs.SR_user
r"""
the value is specified by the user
"""

SR_auto = _ida_segregs.SR_auto
r"""
the value is determined by IDA
"""

SR_autostart = _ida_segregs.SR_autostart
r"""
used as SR_auto for segment starting address
"""



def get_sreg(*args) -> "sel_t":
    r"""
    get_sreg(ea, rg) -> sel_t
    Get value of a segment register. This function uses segment register range and
    default segment register values stored in the segment structure.

    @param ea: (C++: ea_t) linear address in the program
    @param rg: (C++: int) number of the segment register
    @return: value of the segment register, BADSEL if value is unknown.
    """
    return _ida_segregs.get_sreg(*args)

def split_sreg_range(*args) -> "bool":
    r"""
    split_sreg_range(ea, rg, v, tag, silent=False) -> bool
    Create a new segment register range. This function is used when the IDP emulator
    detects that a segment register changes its value.

    @param ea: (C++: ea_t) linear address where the segment register will have a new value. if
               ea==BADADDR, nothing to do.
    @param rg: (C++: int) the number of the segment register
    @param v: (C++: sel_t) the new value of the segment register. If the value is unknown, you
              should specify BADSEL.
    @param tag: (C++: uchar) the register info tag. see Segment register range tags
    @param silent: (C++: bool) if false, display a warning() in the case of failure
    @return: success
    """
    return _ida_segregs.split_sreg_range(*args)

def set_default_sreg_value(*args) -> "bool":
    r"""
    set_default_sreg_value(sg, rg, value) -> bool
    Set default value of a segment register for a segment.

    @param sg: (C++: segment_t *) pointer to segment structure if nullptr, then set the register for
               all segments
    @param rg: (C++: int) number of segment register
    @param value: (C++: sel_t) its default value. this value will be used by get_sreg() if value
                  of the register is unknown at the specified address.
    @return: success
    """
    return _ida_segregs.set_default_sreg_value(*args)

def set_sreg_at_next_code(*args) -> "void":
    r"""
    set_sreg_at_next_code(ea1, ea2, rg, value)
    Set the segment register value at the next instruction. This function is
    designed to be called from idb_event::sgr_changed handler in order to contain
    the effect of changing a segment register value only until the next instruction.

    It is useful, for example, in the ARM module: the modification of the T register
    does not affect existing instructions later in the code.

    @param ea1: (C++: ea_t) address to start to search for an instruction
    @param ea2: (C++: ea_t) the maximal address
    @param rg: (C++: int) the segment register number
    @param value: (C++: sel_t) the segment register value
    """
    return _ida_segregs.set_sreg_at_next_code(*args)

def get_sreg_range(*args) -> "bool":
    r"""
    get_sreg_range(out, ea, rg) -> bool
    Get segment register range by linear address.

    @param out: (C++: sreg_range_t *) segment register range
    @param ea: (C++: ea_t) any linear address in the program
    @param rg: (C++: int) the segment register number
    @return: success
    """
    return _ida_segregs.get_sreg_range(*args)

def get_prev_sreg_range(*args) -> "bool":
    r"""
    get_prev_sreg_range(out, ea, rg) -> bool
    Get segment register range previous to one with address.
    @note: more efficient then get_sreg_range(reg, ea-1)

    @param out: (C++: sreg_range_t *) segment register range
    @param ea: (C++: ea_t) any linear address in the program
    @param rg: (C++: int) the segment register number
    @return: success
    """
    return _ida_segregs.get_prev_sreg_range(*args)

def set_default_dataseg(*args) -> "void":
    r"""
    set_default_dataseg(ds_sel)
    Set default value of DS register for all segments.

    @param ds_sel: (C++: sel_t)
    """
    return _ida_segregs.set_default_dataseg(*args)

def get_sreg_ranges_qty(*args) -> "size_t":
    r"""
    get_sreg_ranges_qty(rg) -> size_t
    Get number of segment register ranges.

    @param rg: (C++: int) the segment register number
    """
    return _ida_segregs.get_sreg_ranges_qty(*args)

def getn_sreg_range(*args) -> "bool":
    r"""
    getn_sreg_range(out, rg, n) -> bool
    Get segment register range by its number.

    @param out: (C++: sreg_range_t *) segment register range
    @param rg: (C++: int) the segment register number
    @param n: (C++: int) number of range (0..qty()-1)
    @return: success
    """
    return _ida_segregs.getn_sreg_range(*args)

def get_sreg_range_num(*args) -> "int":
    r"""
    get_sreg_range_num(ea, rg) -> int
    Get number of segment register range by address.

    @param ea: (C++: ea_t) any address in the range
    @param rg: (C++: int) the segment register number
    @return: -1 if no range occupies the specified address. otherwise returns number
             of the specified range (0..get_srranges_qty()-1)
    """
    return _ida_segregs.get_sreg_range_num(*args)

def del_sreg_range(*args) -> "bool":
    r"""
    del_sreg_range(ea, rg) -> bool
    Delete segment register range started at ea. When a segment register range is
    deleted, the previous range is extended to cover the empty space. The segment
    register range at the beginning of a segment cannot be deleted.

    @param ea: (C++: ea_t) start_ea of the deleted range
    @param rg: (C++: int) the segment register number
    @return: success
    """
    return _ida_segregs.del_sreg_range(*args)

def copy_sreg_ranges(*args) -> "void":
    r"""
    copy_sreg_ranges(dst_rg, src_rg, map_selector=False)
    Duplicate segment register ranges.

    @param dst_rg: (C++: int) number of destination segment register
    @param src_rg: (C++: int) copy ranges from
    @param map_selector: (C++: bool) map selectors to linear addresses using sel2ea()
    """
    return _ida_segregs.copy_sreg_ranges(*args)






# Copyright (c) 2021 Riverbank Computing Limited <info@riverbankcomputing.com>
# 
# This file is part of PyQt5.
# 
# This software is licensed for use under the terms of the Riverbank Commercial
# License.  See the file LICENSE for more details.  It is supplied WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.


# Support PyQt5 sub-packages that have been created by setuptools.
__path__ = __import__('pkgutil').extend_path(__path__, __name__)


def find_qt():
    import os, sys

    qtcore_dll = '\\Qt5Core.dll'

    dll_dir = os.path.dirname(sys.executable)
    if not os.path.isfile(dll_dir + qtcore_dll):
        path = os.environ['PATH']

        dll_dir = os.path.dirname(__file__) + '\\Qt5\\bin'
        if os.path.isfile(dll_dir + qtcore_dll):
            path = dll_dir + ';' + path
            os.environ['PATH'] = path
        else:
            for dll_dir in path.split(';'):
                if os.path.isfile(dll_dir + qtcore_dll):
                    break
            else:
                return

    try:
        os.add_dll_directory(dll_dir)
    except AttributeError:
        pass


find_qt()
del find_qt



"""
summary: drawing custom graphs

description:
  Showing custom graphs, using `ida_graph.GraphViewer`. In addition,
  show how to write actions that can be performed on those.

keywords: graph, actions
"""

# -----------------------------------------------------------------------
# This is an example illustrating how to use the user graphing functionality
# in Python
# (c) Hex-Rays
#

import ida_kernwin
import ida_graph
import ida_ua
import ida_idp
import ida_funcs
import ida_xref

import idautils

class _base_graph_action_handler_t(ida_kernwin.action_handler_t):
    def __init__(self, graph):
        ida_kernwin.action_handler_t.__init__(self)
        self.graph = graph

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_ALWAYS


class GraphCloser(_base_graph_action_handler_t):
    def activate(self, ctx):
        self.graph.Close()


class ColorChanger(_base_graph_action_handler_t):
    def activate(self, ctx):
        self.graph.color = self.graph.color ^ 0xffffff
        self.graph.Refresh()
        return 1


class SelectionPrinter(_base_graph_action_handler_t):
    def activate(self, ctx):
        try:
            sel = ctx.graph_selection
        except:
            # IDA < 7.4 doesn't provide graph selection as part of
            # the action_activation_ctx_t; it needs to be queried.
            sel = ida_graph.screen_graph_selection_t()
            gv = ida_graph.get_graph_viewer(self.graph.GetWidget())
            ida_graph.viewer_get_selection(gv, sel)
        if sel:
            for s in sel:
                if s.is_node:
                    print("Selected node %d" % s.node)
                else:
                    print("Selected edge %d -> %d" % (s.elp.e.src, s.elp.e.dst))
        return 1


class MyGraph(ida_graph.GraphViewer):
    def __init__(self, funcname, result):
        self.title = "call graph of " + funcname
        ida_graph.GraphViewer.__init__(self, self.title)
        self.funcname = funcname
        self.result = result
        self.color = 0xff00ff

        #
        # for the sake of this example, here's how one can use
        # 'ida_kernwin.View_Hooks' (which can be used with all
        # "listing-like" and "graph" widgets) to be notified
        # of cursor movement, current node changes, etc...
        # in this graph.
        #
        class my_view_hooks_t(ida_kernwin.View_Hooks):
            def __init__(self, v):
                ida_kernwin.View_Hooks.__init__(self)
                self.hook()
                # let's use weakrefs, so as soon as the last ref to
                # the 'MyGraph' instance is dropped, the 'my_view_hooks_t'
                # instance hooks can be automatically un-hooked, and deleted.
                # (in other words: avoid circular reference.)
                import weakref
                self.v = weakref.ref(v)

            def view_loc_changed(self, w, now, was):
                now_node = now.renderer_info().pos.node
                was_node = was.renderer_info().pos.node
                if now_node != was_node:
                    if self.v().GetWidget() == w:
                        print("Current node now: #%d (was #%d)" % (now_node, was_node))

        self.my_view_hooks = my_view_hooks_t(self)


    def OnRefresh(self):
        self.Clear()
        id = self.AddNode((self.funcname, self.color))
        for x in self.result:
            callee = self.AddNode((x, self.color))
            self.AddEdge(id, callee)

        return True

    def OnGetText(self, node_id):
        return self[node_id]

    def OnPopup(self, widget, popup_handle):
        # graph closer
        actname = "graph_closer:%s" % self.title
        desc = ida_kernwin.action_desc_t(actname, "Close: %s" % self.title, GraphCloser(self))
        ida_kernwin.attach_dynamic_action_to_popup(None, popup_handle, desc)

        # color changer
        actname = "color_changer:%s" % self.title
        desc = ida_kernwin.action_desc_t(actname, "Change colors: %s" % self.title, ColorChanger(self))
        ida_kernwin.attach_dynamic_action_to_popup(None, popup_handle, desc)

        # selection printer
        actname = "selection_printer:%s" % self.title
        desc = ida_kernwin.action_desc_t(actname, "Print selection: %s" % self.title, SelectionPrinter(self))
        ida_kernwin.attach_dynamic_action_to_popup(None, popup_handle, desc)


def show_graph():
    f = ida_funcs.get_func(ida_kernwin.get_screen_ea())
    if not f:
        print("Must be in a function")
        return
    # Iterate through all function instructions and take only call instructions
    result = []
    tmp = ida_ua.insn_t()
    for x in [x for x in f if (ida_ua.decode_insn(tmp, x) and ida_idp.is_call_insn(tmp))]:
        xb = ida_xref.xrefblk_t()
        for xref in xb.refs_from(x, ida_xref.XREF_FAR):
            if not xref.iscode: continue
            t = ida_funcs.get_func_name(xref.to)
            if not t:
                t = hex(xref.to)
            result.append(t)
    g = MyGraph(ida_funcs.get_func_name(f.start_ea), result)
    if g.Show():
        return g
    else:
        return None

g = show_graph()
if g:
    print("Graph created and displayed!")



"""
Functions that deal with the string list.

While the kernel keeps the string list, it does not update it. The string list
is not used by the kernel because keeping it up-to-date would slow down IDA
without any benefit. If the string list is not cleared using clear_strlist(),
the list will be saved to the database and restored on the next startup.

The users of this list should call build_strlist() if they need an up-to-date
version."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_strlist
else:
    import _ida_strlist

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_strlist.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

class strwinsetup_t(object):
    r"""
    Proxy of C++ strwinsetup_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    minlen = property(_ida_strlist.strwinsetup_t_minlen_get, _ida_strlist.strwinsetup_t_minlen_set, doc=r"""minlen""")
    display_only_existing_strings = property(_ida_strlist.strwinsetup_t_display_only_existing_strings_get, _ida_strlist.strwinsetup_t_display_only_existing_strings_set, doc=r"""display_only_existing_strings""")
    only_7bit = property(_ida_strlist.strwinsetup_t_only_7bit_get, _ida_strlist.strwinsetup_t_only_7bit_set, doc=r"""only_7bit""")
    ignore_heads = property(_ida_strlist.strwinsetup_t_ignore_heads_get, _ida_strlist.strwinsetup_t_ignore_heads_set, doc=r"""ignore_heads""")

    def _get_strtypes(self, *args) -> "PyObject *":
        r"""_get_strtypes(self) -> PyObject *"""
        return _ida_strlist.strwinsetup_t__get_strtypes(self, *args)

    def _set_strtypes(self, *args) -> "PyObject *":
        r"""
        _set_strtypes(self, py_t) -> PyObject *

        Parameters
        ----------
        py_t: PyObject *

        """
        return _ida_strlist.strwinsetup_t__set_strtypes(self, *args)

    strtypes = property(_get_strtypes, _set_strtypes)


    def __init__(self, *args):
        r"""
        __init__(self) -> strwinsetup_t
        """
        _ida_strlist.strwinsetup_t_swiginit(self, _ida_strlist.new_strwinsetup_t(*args))
    __swig_destroy__ = _ida_strlist.delete_strwinsetup_t

# Register strwinsetup_t in _ida_strlist:
_ida_strlist.strwinsetup_t_swigregister(strwinsetup_t)

class string_info_t(object):
    r"""
    Proxy of C++ string_info_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ea = property(_ida_strlist.string_info_t_ea_get, _ida_strlist.string_info_t_ea_set, doc=r"""ea""")
    length = property(_ida_strlist.string_info_t_length_get, _ida_strlist.string_info_t_length_set, doc=r"""length""")
    type = property(_ida_strlist.string_info_t_type_get, _ida_strlist.string_info_t_type_set, doc=r"""type""")

    def __init__(self, *args):
        r"""
        __init__(self, _ea=BADADDR) -> string_info_t

        @param _ea: ea_t
        """
        _ida_strlist.string_info_t_swiginit(self, _ida_strlist.new_string_info_t(*args))

    def __lt__(self, *args) -> "bool":
        r"""
        __lt__(self, r) -> bool

        @param r: string_info_t const &
        """
        return _ida_strlist.string_info_t___lt__(self, *args)
    __swig_destroy__ = _ida_strlist.delete_string_info_t

# Register string_info_t in _ida_strlist:
_ida_strlist.string_info_t_swigregister(string_info_t)


def get_strlist_options(*args) -> "strwinsetup_t const *":
    r"""
    get_strlist_options() -> strwinsetup_t
    Get the static string list options.
    """
    return _ida_strlist.get_strlist_options(*args)

def build_strlist(*args) -> "void":
    r"""
    build_strlist()
    Rebuild the string list.
    """
    return _ida_strlist.build_strlist(*args)

def clear_strlist(*args) -> "void":
    r"""
    clear_strlist()
    Clear the string list.
    """
    return _ida_strlist.clear_strlist(*args)

def get_strlist_qty(*args) -> "size_t":
    r"""
    get_strlist_qty() -> size_t
    Get number of elements in the string list. The list will be loaded from the
    database (if saved) or built from scratch.
    """
    return _ida_strlist.get_strlist_qty(*args)

def get_strlist_item(*args) -> "bool":
    r"""
    get_strlist_item(si, n) -> bool
    Get nth element of the string list (n=0..get_strlist_qty()-1)

    @param si: (C++: string_info_t *)
    @param n: (C++: size_t)
    """
    return _ida_strlist.get_strlist_item(*args)






"""
summary: adding custom menus to IDA

description:
  It is possible to add custom menus to IDA, either at the
  toplevel (i.e., into the menubar), or as submenus of existing
  menus.

  Notes:

    * the same action can be present in more than 1 menu
    * this example does not deal with context menus

keywords: actions
"""

import ida_kernwin

# Create custom menus
ida_kernwin.create_menu("MyToplevelMenu", "&Custom menu", "View")
ida_kernwin.create_menu("MySubMenu", "Custom s&ubmenu", "View/Print internal flags")

# Create some actions
class greeter_t(ida_kernwin.action_handler_t):
    def __init__(self, greetings):
        ida_kernwin.action_handler_t.__init__(self)
        self.greetings = greetings

    def activate(self, ctx):
        print(self.greetings)

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_ALWAYS

ACTION_NAME_0 = "my_action_0"
ACTION_NAME_1 = "my_action_1"
for action_name, greetings in [
        (ACTION_NAME_0, "Hello, world"),
        (ACTION_NAME_1, "Hi there"),
]:
    desc = ida_kernwin.action_desc_t(
        action_name, "Say \"%s\"" % greetings, greeter_t(greetings))
    if ida_kernwin.register_action(desc):
        print("Registered action \"%s\"" % action_name)


# Then, let's attach some actions to them - both core actions
# and custom ones is allowed (also, any action can be attached
# to multiple menus.)
for action_name, path in [
        (ACTION_NAME_0, "Custom menu"),
        (ACTION_NAME_0, "View/Custom submenu/"),
        (ACTION_NAME_1, "Custom menu"),
        (ACTION_NAME_1, "View/Custom submenu/"),
        ("About", "Custom menu"),
        ("About", "View/Custom submenu/"),
]:
    ida_kernwin.attach_action_to_menu(
        path,
        action_name,
        ida_kernwin.SETMENU_INS)



"""
summary: create custom listings in IDA

description:
  How to create simple listings, that will share many of the features
  as the built-in IDA widgets (highlighting, copy & paste,
  notifications, ...)

  In addition, creates actions that will be bound to the
  freshly-created widget (using `ida_kernwin.attach_action_to_popup`.)

keywords: listing, actions
"""

import ida_kernwin
import ida_lines

# -----------------------------------------------------------------------
class say_something_handler_t(ida_kernwin.action_handler_t):
    def __init__(self, thing):
        ida_kernwin.action_handler_t.__init__(self)
        self.thing = thing

    def activate(self, ctx):
        print(self.thing)

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_ALWAYS

    @staticmethod
    def compose_action_name(v):
        return "custview:say_%s" % v


# create actions
actions_variants = ["Hello", "World"]
for av in actions_variants:
    actname = say_something_handler_t.compose_action_name(av)
    if ida_kernwin.unregister_action(actname):
        print("Unregistered previously-registered action \"%s\"" % actname)

    desc = ida_kernwin.action_desc_t(actname, "Say %s" % av, say_something_handler_t(av))
    if ida_kernwin.register_action(desc):
        print("Registered action \"%s\"" % actname)



# -----------------------------------------------------------------------
class mycv_t(ida_kernwin.simplecustviewer_t):
    def Create(self, sn=None, use_colors=True):
        # Form the title
        title = "Simple custom view test"
        if sn:
            title += " %d" % sn
        self.use_colors = use_colors

        # Create the customviewer
        if not ida_kernwin.simplecustviewer_t.Create(self, title):
            return False

        for i in range(0, 100):
            prefix, bg = ida_lines.COLOR_DEFAULT, None
            # make every 10th line a bit special
            if i % 10 == 0:
                prefix = ida_lines.COLOR_DNAME   # i.e., dark yellow...
                bg = 0xFFFF00                 # ...on cyan
            pfx = ida_lines.COLSTR("%3d" % i, ida_lines.SCOLOR_PREFIX)
            if self.use_colors:
                self.AddLine("%s: Line %d" % (pfx, i), fgcolor=prefix, bgcolor=bg)
            else:
                self.AddLine("%s: Line %d" % (pfx, i))

        return True

    def OnClick(self, shift):
        """
        User clicked in the view
        @param shift: Shift flag
        @return: Boolean. True if you handled the event
        """
        print("OnClick, shift=%d" % shift)
        return True

    def OnDblClick(self, shift):
        """
        User dbl-clicked in the view
        @param shift: Shift flag
        @return: Boolean. True if you handled the event
        """
        word = self.GetCurrentWord()
        if not word: word = "<None>"
        print("OnDblClick, shift=%d, current word=%s" % (shift, word))
        return True

    def OnCursorPosChanged(self):
        """
        Cursor position changed.
        @return: Nothing
        """
        print("OnCurposChanged")

    def OnClose(self):
        """
        The view is closing. Use this event to cleanup.
        @return: Nothing
        """
        print("OnClose " + self.title)

    def OnKeydown(self, vkey, shift):
        """
        User pressed a key
        @param vkey: Virtual key code
        @param shift: Shift flag
        @return: Boolean. True if you handled the event
        """
        print("OnKeydown, vk=%d shift=%d" % (vkey, shift))
        # ESCAPE?
        if vkey == 27:
            self.Close()
        # VK_DELETE
        elif vkey == 46:
            n = self.GetLineNo()
            if n is not None:
                self.DelLine(n)
                self.Refresh()
                print("Deleted line %d" % n)
        # Goto?
        elif vkey == ord('G'):
            n = self.GetLineNo()
            if n is not None:
                v = ida_kernwin.ask_long(self.GetLineNo(), "Where to go?")
                if v:
                    self.Jump(v, 0, 5)
        elif vkey == ord('R'):
            print("refreshing....")
            self.Refresh()
        elif vkey == ord('C'):
            print("refreshing current line...")
            self.RefreshCurrent()
        elif vkey == ord('A'):
            s = ida_kernwin.ask_str("NewLine%d" % self.Count(), 0, "Append new line")
            self.AddLine(s)
            self.Refresh()
        elif vkey == ord('X'):
            print("Clearing all lines")
            self.ClearLines()
            self.Refresh()
        elif vkey == ord('I'):
            n = self.GetLineNo()
            s = ida_kernwin.ask_str("InsertedLine%d" % n, 0, "Insert new line")
            self.InsertLine(n, s)
            self.Refresh()
        elif vkey == ord('E'):
            l = self.GetCurrentLine(notags=1)
            if not l:
                return False
            n = self.GetLineNo()
            print("curline=<%s>" % l)
            l = l + ida_lines.COLSTR("*", ida_lines.SCOLOR_VOIDOP)
            self.EditLine(n, l)
            self.RefreshCurrent()
            print("Edited line %d" % n)
        else:
            return False
        return True

    def OnHint(self, lineno):
        """
        Hint requested for the given line number.
        @param lineno: The line number (zero based)
        @return:
            - tuple(number of important lines, hint string)
            - None: if no hint available
        """
        return (1, "OnHint, line=%d" % lineno)

    def Show(self, *args):
        ok = ida_kernwin.simplecustviewer_t.Show(self, *args)
        if ok:
            # permanently attach actions to this viewer's popup menu
            for av in actions_variants:
                actname = say_something_handler_t.compose_action_name(av)
                ida_kernwin.attach_action_to_popup(self.GetWidget(), None, actname)
        return ok

# -----------------------------------------------------------------------
try:
    # created already?
    mycv
    print("Already created, will close it...")
    mycv.Close()
    del mycv
except:
    pass

def show_win():
    x = mycv_t()
    if not x.Create():
        print("Failed to create!")
        return None
    x.Show()
    tcc = x.GetWidget()
    return x

mycv = show_win()
if not mycv:
    del mycv

def make_many(n):
    L = []
    for i in range(1, n+1):
        v = mycv_t()
        if not v.Create(i):
            break
        v.Show()
        L.append(v)
    return L



#############################################################################
##
## Copyright (c) 2021 Riverbank Computing Limited <info@riverbankcomputing.com>
## 
## This file is part of PyQt5.
## 
## This software is licensed for use under the terms of the Riverbank Commercial
## License.  See the file LICENSE for more details.  It is supplied WITHOUT ANY
## WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
## A PARTICULAR PURPOSE.
##
#############################################################################


# A translation table for converting ASCII lower case to upper case.
_ascii_trans_table = bytes.maketrans(b'abcdefghijklmnopqrstuvwxyz',
        b'ABCDEFGHIJKLMNOPQRSTUVWXYZ')


# Convert a string to ASCII upper case irrespective of the current locale.
def ascii_upper(s):
    return s.translate(_ascii_trans_table)



"""
summary: Non-trivial uses of the `ida_kernwin.Form` helper class

description:
  How to query for complex user input, using IDA's built-in forms.

  Note: while this example produces full-fledged forms for complex input,
  simpler types of inputs might can be retrieved by using
  `ida_kernwin.ask_str` and similar functions.

keywords: forms
"""

# -----------------------------------------------------------------------
# This is an example illustrating how to use the Form class
# (c) Hex-Rays
#
import ida_kernwin

# --------------------------------------------------------------------------
class busy_form_t(ida_kernwin.Form):

    class test_chooser_t(ida_kernwin.Choose):
        """
        A simple chooser to be used as an embedded chooser
        """
        def __init__(self, title, nb=5, flags=ida_kernwin.Choose.CH_MULTI):
            ida_kernwin.Choose.__init__(
                self,
                title,
                [
                    ["Address", 10],
                    ["Name", 30]
                ],
                flags=flags,
                embedded=True,
                width=30,
                height=6)
            self.items = [ [str(x), "func_%04d" % x] for x in range(nb + 1) ]
            self.icon = 5

        def OnGetLine(self, n):
            print("getline %d" % n)
            return self.items[n]

        def OnGetSize(self):
            n = len(self.items)
            print("getsize -> %d" % n)
            return n

    def __init__(self):
        self.invert = False
        F = ida_kernwin.Form
        F.__init__(
            self,
            r"""STARTITEM {id:rNormal}
BUTTON YES* Yeah
BUTTON NO Nope
BUTTON CANCEL Nevermind
Form Test

{FormChangeCb}
This is a string:  |+{cStr1}+
This is an address:|+{cAddr1}+
This is some HTML: |+{cHtml1}+
This is a number:  |+{cVal1}+

<#Hint1#Enter text  :{iStr1}>
<#Hint2#Select color:{iColor1}>
Browse test
<#Select a file to open#Browse to open:{iFileOpen}>
<#Select a file to save#Browse to save:{iFileSave}>
<#Select dir#Browse for dir:{iDir}>
Misc
<##Enter a selector value:{iSegment}>
<##Enter a raw hex       :{iRawHex}>
<##Enter a character     :{iChar}>
<##Enter an address      :{iAddr}>
<##Write a type name     :{iType}>
Button test: <##Button1:{iButton1}> <##Button2:{iButton2}>

<##Check boxes##Error output:{rError}> | <##Radio boxes##Green:{rGreen}>
<Normal output:{rNormal}>              | <Red:{rRed}>
<Warnings:{rWarnings}>{cGroup1}>       | <Blue:{rBlue}>{cGroup2}>

<Embedded chooser:{cEChooser}>
The end!
""", {
            'cStr1': F.StringLabel("Hello"),
            'cHtml1': F.StringLabel("<span style='color: red'>Is this red?<span>", tp=F.FT_HTML_LABEL),
            'cAddr1': F.NumericLabel(0x401000, F.FT_ADDR),
            'cVal1' : F.NumericLabel(99, F.FT_HEX),
            'iStr1': F.StringInput(),
            'iColor1': F.ColorInput(),
            'iFileOpen': F.FileInput(open=True),
            'iFileSave': F.FileInput(save=True),
            'iDir': F.DirInput(),
            'iType': F.StringInput(tp=F.FT_TYPE),
            'iSegment': F.NumericInput(tp=F.FT_SEG),
            'iRawHex': F.NumericInput(tp=F.FT_RAWHEX),
            'iAddr': F.NumericInput(tp=F.FT_ADDR),
            'iChar': F.NumericInput(tp=F.FT_CHAR),
            'iButton1': F.ButtonInput(self.OnButton1),
            'iButton2': F.ButtonInput(self.OnButton2),
            'cGroup1': F.ChkGroupControl(("rNormal", "rError", "rWarnings")),
            'cGroup2': F.RadGroupControl(("rRed", "rGreen", "rBlue")),
            'FormChangeCb': F.FormChangeCb(self.OnFormChange),
            'cEChooser' : F.EmbeddedChooserControl(busy_form_t.test_chooser_t("E1"))
        })

    def OnButton1(self, code=0):
        print("Button1 pressed")

    def OnButton2(self, code=0):
        print("Button2 pressed")

    def OnFormChange(self, fid):
        if fid == self.iButton1.id:
            print("Button1 fchg;inv=%s" % self.invert)
            self.SetFocusedField(self.rNormal)
            self.EnableField(self.rError, self.invert)
            self.invert = not self.invert
        elif fid == self.iButton2.id:
            g1 = self.GetControlValue(self.cGroup1)
            g2 = self.GetControlValue(self.cGroup2)
            d = self.GetControlValue(self.iDir)
            f = self.GetControlValue(self.iFileOpen)
            print("cGroup2:%x;Dir=%s;fopen=%s;cGroup1:%x" % (g1, d, f, g2))
        elif fid == self.cEChooser.id:
            l = self.GetControlValue(self.cEChooser)
            print("Chooser: %s" % l)
        elif fid in [self.rGreen.id, self.rRed.id, self.rBlue.id]:
            color = {
                self.rGreen.id : 0x00FF00,
                self.rRed.id   : 0x0000FF,
                self.rBlue.id  : 0xFF0000,
            }
            self.SetControlValue(self.iColor1, color[fid])
        elif fid == self.iColor1.id:
            print("Color changed: %06x" % self.GetControlValue(self.iColor1))
        else:
            print(">>fid:%d" % fid)
        return 1

    @staticmethod
    def compile_and_fiddle_with_fields():
        f = busy_form_t()
        f, args = f.Compile()
        print(args[0])
        print(args[1:])
        f.rNormal.checked = True
        f.rWarnings.checked = True
        print(hex(f.cGroup1.value))

        f.rGreen.selected = True
        print(f.cGroup2.value)
        print("Title: '%s'" % f.title)

        f.Free()

    @staticmethod
    def test():
        f = busy_form_t()

        # Compile (in order to populate the controls)
        f.Compile()

        f.iColor1.value = 0x5bffff
        f.iDir.value = os.getcwd()
        f.iChar.value = ord("a")
        f.rNormal.checked = True
        f.rWarnings.checked = True
        f.rGreen.selected = True
        f.iStr1.value = "Hello"
        f.iFileSave.value = "*.*"
        f.iFileOpen.value = "*.*"

        # Execute the form
        ok = f.Execute()
        print("r=%d" % ok)
        if ok == 1:
            print("f.str1=%s" % f.iStr1.value)
            print("f.color1=%x" % f.iColor1.value)
            print("f.openfile=%s" % f.iFileOpen.value)
            print("f.savefile=%s" % f.iFileSave.value)
            print("f.dir=%s" % f.iDir.value)
            print("f.type=%s" % f.iType.value)
            print("f.seg=%s" % f.iSegment.value)
            print("f.rawhex=%x" % f.iRawHex.value)
            print("f.char=%x" % f.iChar.value)
            print("f.addr=%x" % f.iAddr.value)
            print("f.cGroup1=%x" % f.cGroup1.value)
            print("f.cGroup2=%x" % f.cGroup2.value)
            sel = f.cEChooser.selection
            if sel is None:
                print("No selection")
            else:
                print("Selection: %s" % sel)

        # Dispose the form
        f.Free()

# --------------------------------------------------------------------------
class multiline_text_t(ida_kernwin.Form):
    """Simple Form to test multilinetext"""
    def __init__(self):
        F = ida_kernwin.Form
        F.__init__(self, r"""STARTITEM 0
BUTTON YES* Yeah
BUTTON NO Nope
BUTTON CANCEL NONE
Form Test

{FormChangeCb}
<Multilinetext:{txtMultiLineText}>
""", {
            'txtMultiLineText': F.MultiLineTextControl(text="Hello"),
            'FormChangeCb': F.FormChangeCb(self.OnFormChange),
        })

    def OnFormChange(self, fid):
        if fid == self.txtMultiLineText.id:
            pass
        elif fid == -2:
            ti = self.GetControlValue(self.txtMultiLineText)
            print("ti.text = %s" % ti.text)
        else:
            print(">>fid:%d" % fid)
        return 1

    @staticmethod
    def test(execute=True):
        f = multiline_text_t()
        f, args = f.Compile()
        if execute:
            ok = f.Execute()
        else:
            print(args[0])
            print(args[1:])
            ok = 0
        if ok == 1:
            assert f.txtMultiLineText.text == f.txtMultiLineText.value
            print(f.txtMultiLineText.text)
        f.Free()


# --------------------------------------------------------------------------
class multiline_text_and_dropdowns_t(ida_kernwin.Form):
    """Simple Form to test multilinetext and combo box controls"""
    def __init__(self):
        self.__n = 0
        F = ida_kernwin.Form
        F.__init__(self,
r"""BUTTON YES* Yeah
BUTTON NO Nope
BUTTON CANCEL NONE
Dropdown list test

{FormChangeCb}
<Dropdown list (readonly):{cbReadonly}> <Add element:{iButtonAddelement}> <Set index:{iButtonSetIndex}>
<Dropdown list (editable):{cbEditable}> <Set string:{iButtonSetString}>
""", {
            'FormChangeCb': F.FormChangeCb(self.OnFormChange),
            'cbReadonly': F.DropdownListControl(
                        items=["red", "green", "blue"],
                        readonly=True,
                        selval=1),
            'cbEditable': F.DropdownListControl(
                        items=["1MB", "2MB", "3MB", "4MB"],
                        readonly=False,
                        selval="4MB"),
            'iButtonAddelement': F.ButtonInput(self.OnButtonNop),
            'iButtonSetIndex': F.ButtonInput(self.OnButtonNop),
            'iButtonSetString': F.ButtonInput(self.OnButtonNop),
        })


    def OnButtonNop(self, code=0):
        """Do nothing, we will handle events in the form callback"""
        pass

    def OnFormChange(self, fid):
        if fid == self.iButtonSetString.id:
            s = ida_kernwin.ask_str("none", 0, "Enter value")
            if s:
                self.SetControlValue(self.cbEditable, s)
        elif fid == self.iButtonSetIndex.id:
            s = ida_kernwin.ask_str("1", 0, "Enter index value:")
            if s:
                try:
                    i = int(s)
                except:
                    i = 0
                self.SetControlValue(self.cbReadonly, i)
        elif fid == self.iButtonAddelement.id:
            # add a value to the string list
            self.__n += 1
            self.cbReadonly.add("some text #%d" % self.__n)
            # Refresh the control
            self.RefreshField(self.cbReadonly)
        elif fid == -2:
            s = self.GetControlValue(self.cbEditable)
            print("user entered: %s" % s)
            sel_idx = self.GetControlValue(self.cbReadonly)
        return 1

    @staticmethod
    def test(execute=True):
        f = multiline_text_and_dropdowns_t()
        f, args = f.Compile()
        if execute:
            ok = f.Execute()
        else:
            print(args[0])
            print(args[1:])
            ok = 0
        if ok == 1:
            print("Editable: %s" % f.cbEditable.value)
            print("Readonly: %s" % f.cbReadonly.value)
        f.Free()

    NON_MODAL_INSTANCE = None

    @staticmethod
    def test_non_modal():
        if multiline_text_and_dropdowns_t.NON_MODAL_INSTANCE is None:
            f = multiline_text_and_dropdowns_t()
            f.modal = False
            f.openform_flags = ida_kernwin.PluginForm.FORM_TAB
            f, _ = f.Compile()
            multiline_text_and_dropdowns_t.NON_MODAL_INSTANCE = f
        multiline_text_and_dropdowns_t.NON_MODAL_INSTANCE.Open()

# --------------------------------------------------------------------------
busy_form_t.test()



"""
summary: retrieving & dumping debuggee symbols

description:
  Queries the debugger (possibly remotely) for the list of
  symbols that the process being debugged, provides.
"""

import ida_dbg
import ida_ida
import ida_name

def main():
    if not ida_dbg.is_debugger_on():
        print("Please run the process first!")
        return
    if ida_dbg.get_process_state() != ida_dbg.DSTATE_SUSP:
        print("Please suspend the debugger first!")
        return

    dn = ida_name.get_debug_names(
        ida_ida.inf_get_min_ea(),
        ida_ida.inf_get_max_ea())
    for i in dn:
        print("%08x: %s" % (i, dn[i]))

main()



import os
from datetime import datetime

import ida_lines
import ida_netnode
import idaapi
import idautils
import ida_kernwin

from yaraforge.metadata import pathnames
from .logger import get_global_logger
from ida_lines import GENDSM_REMOVE_TAGS
from pathlib import Path

logger = get_global_logger(pathnames['logger_dir'])


def get_cache_dir():
    """
    Get the cache directory.
    :return: The cache directory.
    """
    logger.info("Getting cache directory.")
    cache_dir = Path(pathnames['cache_dir'])  # Convert string to Path object
    cache_dir.mkdir(parents=True, exist_ok=True)  # This line should now work properly
    return cache_dir


def make_dirs(path_array):
    """
    Make directories for the given paths.
    :param path_array: The paths to make directories for.
    :return: None
    """
    logger.info(f"Making directories for {path_array}.")
    for path in path_array:  # This should be path_array, not pathnames.values()
        try:
            os.makedirs(path, exist_ok=True)
        except PermissionError:
            logger.error(f"Permission denied: Unable to create directory {path}.")
            print(f"Permission denied: Unable to create directory {path}.")
        except Exception as e:
            logger.error(f"Error creating directory {path}: {e}")
            print(f"Error creating directory {path}: {e}")


def get_desktop_path():
    """
    Get the path to the desktop.
    :return: The path to the desktop.
    """
    logger.info("Getting desktop path.")
    desktop_path = os.path.join(os.path.join(os.environ['USERPROFILE']), 'Desktop')
    return desktop_path


def explore_netnodes():
    """
    Explore netnodes to find CAPA netnodes.
    If CAPA netnodes are not found, prompt the user to run CAPA analysis first.
    """
    logger.info("Exploring netnodes.")
    n = ida_netnode.netnode()
    found = False

    if n.start():
        while True:
            name = n.get_name()
            if name and "$ com.mandiant.capa" in name:
                found = True
                logger.info(f"Found CAPA netnode: {name}")
                print(f"Found CAPA netnode: {name}")
            if not n.next():
                if not found:
                    # If no CAPA netnodes are found, inform the user to run CAPA analysis first.
                    logger.error("No CAPA netnodes found. Please run CAPA analysis first.")
                    print("No CAPA netnodes found.")
                    ida_kernwin.warning("No CAPA netnodes found. Please run CAPA analysis first.")
                    raise RuntimeError("No CAPA netnodes found. Please run CAPA analysis first.")
                logger.info("Finished exploring netnodes.")
                print("Finished exploring netnodes.")
                break
    else:
        logger.error("Failed to start netnode exploration.")
        print("Failed to start netnode exploration.")
        ida_kernwin.warning("Failed to start netnode exploration. Please check your IDA setup.")
        raise RuntimeError("Failed to start netnode exploration.")  # Raise an exception if it fails to start


def custom_json_serializer(o):
    """
    Custom JSON serializer for datetime objects.
    :param o: The object to serialize.
    :return: The serialized object.
    """
    if isinstance(o, datetime):
        logger.info("Using custom JSON serializer for datetime object.")
        return o.isoformat()
    logger.error(f"Object of type {o.__class__.__name__} is not JSON serializable.")
    raise TypeError(f"Object of type {o.__class__.__name__} is not JSON serializable.")


def get_instructions_for_address(addr):
    """
    Get the instructions for a given address.
    :param addr:
    :return: A list of instructions.
    """
    instructions = []
    # 確保地址是整數類型
    if isinstance(addr, str):
        addr = int(addr, 16)  # 假設地址是十六進位字符串，轉換為整數
    elif not isinstance(addr, int):
        logger.error(f"Address must be an integer or hex string. Received: {addr}")
        return instructions

    func = idaapi.get_func(addr)
    if func:
        fci = idaapi.FlowChart(func)
        for block in fci:
            if block.start_ea <= addr < block.end_ea:
                for head in idautils.Heads(block.start_ea, block.end_ea):
                    disasm_line = ida_lines.generate_disasm_line(head, GENDSM_REMOVE_TAGS)
                    if disasm_line:
                        instructions.append(f"0x{head:X}: {disasm_line}")
    return instructions



"""
Registry related functions.

IDA uses the registry to store global configuration options that must persist
after IDA has been closed.

On Windows, IDA uses the Windows registry directly. On Unix systems, the
registry is stored in a file (typically ~/.idapro/ida.reg).

The root key for accessing IDA settings in the registry is defined by
ROOT_KEY_NAME."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_registry
else:
    import _ida_registry

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_registry.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi


def reg_read_string(*args) -> "PyObject *":
    r"""
    reg_read_string(name, subkey=None, _def=None) -> str
    Read a string from the registry.

    @param name: (C++: const char *) value name
    @param subkey: (C++: const char *) key name
    @param def: char const *
    @return: success
    """
    return _ida_registry.reg_read_string(*args)

def reg_data_type(*args) -> "regval_type_t":
    r"""
    reg_data_type(name, subkey=None) -> regval_type_t
    Get data type of a given value.

    @param name: (C++: const char *) value name
    @param subkey: (C++: const char *) key name
    @return: false if the [key+]value doesn't exist
    """
    return _ida_registry.reg_data_type(*args)

def reg_read_binary(*args) -> "PyObject *":
    r"""
    reg_read_binary(name, subkey=None) -> bytes or None
    Read binary data from the registry.

    @param name: (C++: const char *) value name
    @param subkey: (C++: const char *) key name
    @return: success
    """
    return _ida_registry.reg_read_binary(*args)

def reg_write_binary(*args) -> "PyObject *":
    r"""
    reg_write_binary(name, py_bytes, subkey=None) -> PyObject *
    Write binary data to the registry.

    @param name: (C++: const char *) value name
    @param py_bytes: PyObject *
    @param subkey: (C++: const char *) key name
    """
    return _ida_registry.reg_write_binary(*args)

def reg_subkey_subkeys(*args) -> "PyObject *":
    r"""
    reg_subkey_subkeys(name) -> [str, ...]
    Get all subkey names of given key.

    @param name: (C++: const char *) char const *
    """
    return _ida_registry.reg_subkey_subkeys(*args)

def reg_subkey_values(*args) -> "PyObject *":
    r"""
    reg_subkey_values(name) -> [str, ...]
    Get all value names under given key.

    @param name: (C++: const char *) char const *
    """
    return _ida_registry.reg_subkey_values(*args)
ROOT_KEY_NAME = _ida_registry.ROOT_KEY_NAME
r"""
Default key used to store IDA settings in registry (Windows version).
@note: this name is automatically prepended to all key names passed to functions
       in this file.
"""

reg_unknown = _ida_registry.reg_unknown
r"""
unknown
"""

reg_sz = _ida_registry.reg_sz
r"""
utf8 string
"""

reg_binary = _ida_registry.reg_binary
r"""
binary data
"""

reg_dword = _ida_registry.reg_dword
r"""
32-bit number
"""


def reg_delete_subkey(*args) -> "bool":
    r"""
    reg_delete_subkey(name) -> bool
    Delete a key from the registry.

    @param name: (C++: const char *) char const *
    """
    return _ida_registry.reg_delete_subkey(*args)

def reg_delete_tree(*args) -> "bool":
    r"""
    reg_delete_tree(name) -> bool
    Delete a subtree from the registry.

    @param name: (C++: const char *) char const *
    """
    return _ida_registry.reg_delete_tree(*args)

def reg_delete(*args) -> "bool":
    r"""
    reg_delete(name, subkey=None) -> bool
    Delete a value from the registry.

    @param name: (C++: const char *) value name
    @param subkey: (C++: const char *) parent key
    @return: success
    """
    return _ida_registry.reg_delete(*args)

def reg_subkey_exists(*args) -> "bool":
    r"""
    reg_subkey_exists(name) -> bool
    Is there already a key with the given name?

    @param name: (C++: const char *) char const *
    """
    return _ida_registry.reg_subkey_exists(*args)

def reg_exists(*args) -> "bool":
    r"""
    reg_exists(name, subkey=None) -> bool
    Is there already a value with the given name?

    @param name: (C++: const char *) value name
    @param subkey: (C++: const char *) parent key
    """
    return _ida_registry.reg_exists(*args)

def reg_read_strlist(*args) -> "qstrvec_t *":
    r"""
    reg_read_strlist(subkey)
    Retrieve all string values associated with the given key. Also see
    reg_update_strlist().

    @param subkey: (C++: const char *) char const *
    """
    return _ida_registry.reg_read_strlist(*args)

def reg_update_strlist(*args) -> "void":
    r"""
    reg_update_strlist(subkey, add, maxrecs, rem=None, ignorecase=False)
    Update list of strings associated with given key.

    @param subkey: (C++: const char *) key name
    @param add: (C++: const char *) string to be added to list, can be nullptr
    @param maxrecs: (C++: size_t) limit list to this size
    @param rem: (C++: const char *) string to be removed from list, can be nullptr
    @param ignorecase: (C++: bool) ignore case for 'add' and 'rem'
    """
    return _ida_registry.reg_update_strlist(*args)

def reg_write_string(*args) -> "void":
    r"""
    reg_write_string(name, utf8, subkey=None)
    Write a string to the registry.

    @param name: (C++: const char *) value name
    @param utf8: (C++: const char *) utf8-encoded string
    @param subkey: (C++: const char *) key name
    """
    return _ida_registry.reg_write_string(*args)

def reg_read_int(*args) -> "int":
    r"""
    reg_read_int(name, defval, subkey=None) -> int
    Read integer value from the registry.

    @param name: (C++: const char *) value name
    @param defval: (C++: int) default value
    @param subkey: (C++: const char *) key name
    @return: the value read from the registry, or 'defval' if the read failed
    """
    return _ida_registry.reg_read_int(*args)

def reg_write_int(*args) -> "void":
    r"""
    reg_write_int(name, value, subkey=None)
    Write integer value to the registry.

    @param name: (C++: const char *) value name
    @param value: (C++: int) value to write
    @param subkey: (C++: const char *) key name
    """
    return _ida_registry.reg_write_int(*args)

def reg_read_bool(*args) -> "bool":
    r"""
    reg_read_bool(name, defval, subkey=None) -> bool
    Read boolean value from the registry.

    @param name: (C++: const char *) value name
    @param defval: (C++: bool) default value
    @param subkey: (C++: const char *) key name
    @return: boolean read from registry, or 'defval' if the read failed
    """
    return _ida_registry.reg_read_bool(*args)

def reg_write_bool(*args) -> "void":
    r"""
    reg_write_bool(name, value, subkey=None)
    Write boolean value to the registry.

    @param name: (C++: const char *) value name
    @param value: (C++: int) boolean to write (nonzero = true)
    @param subkey: (C++: const char *) key name
    """
    return _ida_registry.reg_write_bool(*args)

def reg_update_filestrlist(*args) -> "void":
    r"""
    reg_update_filestrlist(subkey, add, maxrecs, rem=None)
    Update registry with a file list. Case sensitivity will vary depending on the
    target OS.
    @note: 'add' and 'rem' must be UTF-8, just like for regular string operations.

    @param subkey: (C++: const char *) char const *
    @param add: (C++: const char *) char const *
    @param maxrecs: (C++: size_t)
    @param rem: (C++: const char *) char const *
    """
    return _ida_registry.reg_update_filestrlist(*args)

def set_registry_root(*args) -> "bool":
    r"""
    set_registry_root(name) -> bool

    @param name: char const *
    """
    return _ida_registry.set_registry_root(*args)

def reg_load(*args) -> "void":
    r"""
    reg_load()
    """
    return _ida_registry.reg_load(*args)

def reg_flush(*args) -> "void":
    r"""
    reg_flush()
    """
    return _ida_registry.reg_flush(*args)






"""
summary: logging IDB events

description:
  these hooks will be notified about IDB events, and
  dump their information to the "Output" window
"""

import inspect

import ida_idp

class idb_logger_hooks_t(ida_idp.IDB_Hooks):

    def __init__(self):
        ida_idp.IDB_Hooks.__init__(self)
        self.inhibit_log = 0;

    def _format_value(self, v):
        return str(v)

    def _log(self, msg=None):
        if self.inhibit_log <= 0:
            if msg:
                print(">>> idb_logger_hooks_t: %s" % msg)
            else:
                stack = inspect.stack()
                frame, _, _, _, _, _ = stack[1]
                args, _, _, values = inspect.getargvalues(frame)
                method_name = inspect.getframeinfo(frame)[2]
                argstrs = []
                for arg in args[1:]:
                    argstrs.append("%s=%s" % (arg, self._format_value(values[arg])))
                print(">>> idb_logger_hooks_t.%s: %s" % (method_name, ", ".join(argstrs)))
        return 0

    def adding_segm(self, segment):
        return self._log()

    def allsegs_moved(self, info):
        return self._log()

    def auto_empty(self):
        return self._log()

    def auto_empty_finally(self):
        return self._log()

    def bookmark_changed(self, index, pos, desc, op):
        return self._log()

    def byte_patched(self, ea, old_value):
        return self._log()

    def callee_addr_changed(self, ea, callee):
        return self._log()

    def changing_cmt(self, ea, is_repeatable, new_comment):
        return self._log()

    def changing_enum_bf(self, tid, new_bf):
        return self._log()

    def changing_enum_cmt(self, tid, is_repeatable, new_comment):
        return self._log()

    def changing_op_ti(self, ea, n, new_type, new_fnames):
        return self._log()

    def changing_op_type(self, ea, n, opinfo):
        return self._log()

    def changing_range_cmt(self, kind, _range, comment, is_repeatable):
        return self._log()

    def changing_segm_class(self, segment):
        return self._log()

    def changing_segm_end(self, segment, new_end, flags):
        return self._log()

    def changing_segm_name(self, segment, old_name):
        return self._log()

    def changing_segm_start(self, segment, new_start, flags):
        return self._log()

    def changing_struc_align(self, sptr):
        return self._log()

    def changing_struc_cmt(self, tid, is_repeatable, comment):
        return self._log()

    def changing_struc_member(self, sptr, mptr, flags, ti, nbytes):
        return self._log()

    def changing_ti(self, ea, new_type, new_fnames):
        return self._log()

    def closebase(self):
        return self._log()

    def cmt_changed(self, ea, is_repeatable):
        return self._log()

    def compiler_changed(self, may_adjust_inf_fields):
        return self._log()

    def deleting_enum(self, tid):
        return self._log()

    def deleting_enum_member(self, tid, cid):
        return self._log()

    def deleting_func(self, pfn):
        return self._log()

    def deleting_func_tail(self, pfn, tail):
        return self._log()

    def deleting_segm(self, start_ea):
        return self._log()

    def deleting_struc(self, sptr):
        return self._log()

    def deleting_struc_member(self, sptr, mptr):
        return self._log()

    def deleting_tryblks(self, _range):
        return self._log()

    def destroyed_items(self, ea1, ea2, will_disable_range):
        return self._log()

    def determined_main(self, main):
        return self._log()

    def dirtree_link(self, dt, path, is_link):
        return self._log()

    def dirtree_mkdir(self, dt, path):
        return self._log()

    def dirtree_move(self, dt, _from, to):
        return self._log()

    def dirtree_rank(self, dt, path, rank):
        return self._log()

    def dirtree_rmdir(self, dt, path):
        return self._log()

    def dirtree_rminode(self, dt, inode):
        return self._log()

    def dirtree_segm_moved(self, dt):
        return self._log()

    def enum_bf_changed(self, tid):
        return self._log()

    def enum_cmt_changed(self, tid, is_repeatable):
        return self._log()

    def enum_created(self, tid):
        return self._log()

    def enum_deleted(self, tid):
        return self._log()

    def enum_member_created(self, tid, cid):
        return self._log()

    def enum_member_deleted(self, tid, cid):
        return self._log()

    def enum_renamed(self, tid):
        return self._log()

    def expanding_struc(self, sptr, offset, delta):
        return self._log()

    def extlang_changed(self, kind, el, idx):
        return self._log()

    def extra_cmt_changed(self, ea, line_idx, comment):
        return self._log()

    def flow_chart_created(self, fc):
        return self._log()

    def frame_deleted(self, pfn):
        return self._log()

    def func_added(self, pfn):
        return self._log()

    def func_deleted(self, func_ea):
        return self._log()

    def func_noret_changed(self, pfn):
        return self._log()

    def func_tail_appended(self, pfn, tail):
        return self._log()

    def func_tail_deleted(self, pfn, tail_ea):
        return self._log()

    def func_updated(self, pfn):
        return self._log()

    def idasgn_loaded(self, sig_name):
        return self._log()

    def item_color_changed(self, ea, color):
        return self._log()

    def kernel_config_loaded(self, pass_number):
        return self._log()

    def loader_finished(self, li, neflags, filetypename):
        return self._log()

    def local_types_changed(self):
        return self._log()

    def make_code(self, insn):
        return self._log()

    def make_data(self, ea, flags, tid, _len):
        return self._log()

    def op_ti_changed(self, ea, n, _type, fnames):
        return self._log()

    def op_type_changed(self, ea, n):
        return self._log()

    def range_cmt_changed(self, kind, _range, comment, is_repeatable):
        return self._log()

    def renamed(self, ea, new_name, is_local_name, old_name):
        return self._log()

    def renaming_enum(self, tid, is_enum, new_name):
        return self._log()

    def renaming_struc(self, tid, old_name, new_name):
        return self._log()

    def renaming_struc_member(self, sptr, mptr, new_name):
        return self._log()

    def savebase(self):
        return self._log()

    def segm_added(self, segment):
        return self._log()

    def segm_attrs_updated(self, segment):
        return self._log()

    def segm_class_changed(self, segment, sclass):
        return self._log()

    def segm_deleted(self, start_ea, end_ea, flags):
        return self._log()

    def segm_end_changed(self, segment, old_end):
        return self._log()

    def segm_moved(self, _from, to, size, changed_netmap):
        return self._log()

    def segm_name_changed(self, segment, name):
        return self._log()

    def segm_start_changed(self, segment, old_start):
        return self._log()

    def set_func_end(self, pfn, new_end):
        return self._log()

    def set_func_start(self, pfn, new_start):
        return self._log()

    def sgr_changed(self, start_ea, end_ea, regnum, value, old_value, tag):
        return self._log()

    def sgr_deleted(self, start_ea, end_ea, regnum):
        return self._log()

    def stkpnts_changed(self, pfn):
        return self._log()

    def struc_align_changed(self, sptr):
        return self._log()

    def struc_cmt_changed(self, tid, is_repeatable):
        return self._log()

    def struc_created(self, tid):
        return self._log()

    def struc_deleted(self, tid):
        return self._log()

    def struc_expanded(self, sptr):
        return self._log()

    def struc_member_changed(self, sptr, mptr):
        return self._log()

    def struc_member_created(self, sptr, mptr):
        return self._log()

    def struc_member_deleted(self, sptr, mid, offset):
        return self._log()

    def struc_member_renamed(self, sptr, mptr):
        return self._log()

    def struc_renamed(self, sptr, success):
        return self._log()

    def tail_owner_changed(self, tail, owner_func, old_owner):
        return self._log()

    def thunk_func_created(self, pfn):
        return self._log()

    def ti_changed(self, ea, _type, fnames):
        return self._log()

    def tryblks_updated(self, tbv):
        return self._log()

    def updating_tryblks(self, tbv):
        return self._log()

    def upgraded(self, _from):
        return self._log()

    def enum_width_changed(self, tid, width):
        return self._log()

    def enum_flag_changed(self, tid, flags):
        return self._log()

    def enum_ordinal_changed(self, tid, ordinal):
        return self._log()


idb_hooks = idb_logger_hooks_t()
idb_hooks.hook()



"""
summary: list all xrefs to a function stack variable

description:
  Contrary to (in-memory) data & code xrefs, retrieving stack variables
  xrefs requires a bit more work than just using ida_xref's first_to(),
  next_to() (or higher level utilities such as idautils.XrefsTo)

keywords: xrefs
"""

ACTION_NAME = "list_stkvar_xrefs:list"
ACTION_SHORTCUT = "Ctrl+Shift+F7"

import ida_bytes
import ida_frame
import ida_funcs
import ida_ida
import ida_kernwin
import ida_struct
import ida_ua

class list_stkvar_xrefs_ah_t(ida_kernwin.action_handler_t):
    def activate(self, ctx):
        cur_ea = ida_kernwin.get_screen_ea()
        pfn = ida_funcs.get_func(cur_ea)
        if pfn:
            v = ida_kernwin.get_current_viewer()
            result = ida_kernwin.get_highlight(v)
            if result:
                stkvar_name, _ = result
                frame = ida_frame.get_frame(cur_ea)
                sptr = ida_struct.get_struc(frame.id)
                mptr = ida_struct.get_member_by_name(sptr, stkvar_name)
                if mptr:
                    for ea in pfn:
                        F = ida_bytes.get_flags(ea)
                        for n in range(ida_ida.UA_MAXOP):
                            if not ida_bytes.is_stkvar(F, n):
                                continue
                            insn = ida_ua.insn_t()
                            if not ida_ua.decode_insn(insn, ea):
                                continue
                            v = ida_frame.calc_stkvar_struc_offset(pfn, insn, n)
                            if v >= mptr.soff and v < mptr.eoff:
                                print("Found xref at 0x%08x, operand #%d" % (ea, n))
                else:
                    print("No stack variable named \"%s\"" % stkvar_name)
        else:
            print("Please position the cursor within a function")

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET \
            if ctx.widget_type == ida_kernwin.BWN_DISASM \
               else ida_kernwin.AST_DISABLE_FOR_WIDGET

adesc = ida_kernwin.action_desc_t(
    ACTION_NAME,
    "List stack variable xrefs",
    list_stkvar_xrefs_ah_t(),
    ACTION_SHORTCUT)

if ida_kernwin.register_action(adesc):
    print("Action registered. Please press \"%s\" to use" % ACTION_SHORTCUT)



"""
summary: better integrating custom widgets in the desktop layout

description:
  This is an example demonstrating how one can create widgets from a plugin,
  and have them re-created automatically at IDA startup-time or at desktop load-time.

  This example should be placed in the 'plugins' directory of the
  IDA installation, for it to work.

  There are 2 ways to use this example:
  1) reloading an IDB, where the widget was opened
     - open the widget ('View > Open subview > ...')
     - save this IDB, and close IDA
     - restart IDA with this IDB
       => the widget will be visible

  2) reloading a desktop, where the widget was opened
     - open the widget ('View > Open subview > ...')
     - save the desktop ('Windows > Save desktop...') under, say, the name 'with_auto'
     - start another IDA instance with some IDB, and load that desktop
       => the widget will be visible

keywords: desktop
"""

import ida_idaapi
import ida_kernwin

title = "Auto-instantiable at IDA startup"

# -----------------------------------------------------------------------
class auto_inst_t(ida_kernwin.simplecustviewer_t):
    def __init__(self):
        ida_kernwin.simplecustviewer_t.__init__(self)

    def Create(self):
        if not ida_kernwin.simplecustviewer_t.Create(self, title):
            return False

        text = __doc__
        for l in text.split("\n"):
            self.AddLine(l)
        return True

# -----------------------------------------------------------------------
auto_inst = None

# -----------------------------------------------------------------------
def register_open_action():
    """
    Provide the action that will create the widget
    when the user asks for it.
    """
    class create_widget_t(ida_kernwin.action_handler_t):
        def activate(self, ctx):
            if ida_kernwin.find_widget(title) is None:
                global auto_inst
                auto_inst = auto_inst_t()
                assert(auto_inst.Create())
                assert(auto_inst.Show())

        def update(self, ctx):
            return ida_kernwin.AST_ENABLE_ALWAYS

    action_name = "autoinst:create"
    ida_kernwin.register_action(
        ida_kernwin.action_desc_t(
            action_name,
            title,
            create_widget_t()))
    ida_kernwin.attach_action_to_menu(
        "View/Open subviews/Strings",
        action_name,
        ida_kernwin.SETMENU_APP)

# -----------------------------------------------------------------------
auto_inst_hooks = None
def register_autoinst_hooks():
    """
    Register hooks that will create the widget when IDA
    requires it because of the IDB/desktop
    """
    class auto_inst_hooks_t(ida_kernwin.UI_Hooks):
        def create_desktop_widget(self, ttl, cfg):
            if ttl == title:
                global auto_inst
                auto_inst = auto_inst_t()
                assert(auto_inst.Create())
                return auto_inst.GetWidget()

    global auto_inst_hooks
    auto_inst_hooks = auto_inst_hooks_t()
    auto_inst_hooks.hook()

# -----------------------------------------------------------------------
class auto_inst_plugin_t(ida_idaapi.plugin_t):
    flags = 0
    comment = "This plugin creates a widget that will be recreated automatically if needed, either at startup or when loading a desktop that requires it"
    help = "No help, really"
    wanted_name = "autoinst"
    wanted_hotkey = ""

    def init(self):
        register_open_action()
        register_autoinst_hooks()

    def run(self, arg):
        pass

    def term(self):
        pass


def PLUGIN_ENTRY():
    return auto_inst_plugin_t()



"""
summary: code to be run right after IDAPython initialization

description:
  The `idapythonrc.py` file:

    * %APPDATA%\Hex-Rays\IDA Pro\idapythonrc.py (on Windows)
    * ~/.idapro/idapythonrc.py (on Linux & Mac)

  can contain any IDAPython code that will be run as soon as
  IDAPython is done successfully initializing.
"""

# Add your favourite script to ScriptBox for easy access
# scriptbox.addscript("/here/is/my/favourite/script.py")

# Uncomment if you want to set Python as default interpreter in IDA
# import ida_idaapi
# ida_idaapi.enable_extlang_python(True)

# Disable the Python from interactive command-line
# import ida_idaapi
# ida_idaapi.enable_python_cli(False)

# Set the timeout for the script execution cancel dialog
# import ida_idaapi
# ida_idaapi.set_script_timeout(10)



"""
Architecture independent exception handling info.

Try blocks have the following general properties:
* A try block specifies a possibly fragmented guarded code region.
* Each try block has always at least one catch/except block description
* Each catch block contains its boundaries and a filter.
* Additionally a catch block can hold sp adjustment and the offset to the
exception object offset (C++).
* Try blocks can be nested. Nesting is automatically calculated at the retrieval
time.
* There may be (nested) multiple try blocks starting at the same address.

See examples in tests/input/src/eh_tests."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_tryblks
else:
    import _ida_tryblks

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_tryblks.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

import ida_range
class tryblks_t(object):
    r"""
    Proxy of C++ qvector< tryblk_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> tryblks_t
        __init__(self, x) -> tryblks_t

        @param x: qvector< tryblk_t > const &
        """
        _ida_tryblks.tryblks_t_swiginit(self, _ida_tryblks.new_tryblks_t(*args))
    __swig_destroy__ = _ida_tryblks.delete_tryblks_t

    def push_back(self, *args) -> "tryblk_t &":
        r"""
        push_back(self, x)

        @param x: tryblk_t const &

        push_back(self) -> tryblk_t
        """
        return _ida_tryblks.tryblks_t_push_back(self, *args)

    def pop_back(self, *args) -> "void":
        r"""
        pop_back(self)
        """
        return _ida_tryblks.tryblks_t_pop_back(self, *args)

    def size(self, *args) -> "size_t":
        r"""
        size(self) -> size_t
        """
        return _ida_tryblks.tryblks_t_size(self, *args)

    def empty(self, *args) -> "bool":
        r"""
        empty(self) -> bool
        """
        return _ida_tryblks.tryblks_t_empty(self, *args)

    def at(self, *args) -> "tryblk_t const &":
        r"""
        at(self, _idx) -> tryblk_t

        @param _idx: size_t
        """
        return _ida_tryblks.tryblks_t_at(self, *args)

    def qclear(self, *args) -> "void":
        r"""
        qclear(self)
        """
        return _ida_tryblks.tryblks_t_qclear(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_tryblks.tryblks_t_clear(self, *args)

    def resize(self, *args) -> "void":
        r"""
        resize(self, _newsize, x)

        @param _newsize: size_t
        @param x: tryblk_t const &

        resize(self, _newsize)

        @param _newsize: size_t
        """
        return _ida_tryblks.tryblks_t_resize(self, *args)

    def grow(self, *args) -> "void":
        r"""
        grow(self, x=tryblk_t())

        @param x: tryblk_t const &
        """
        return _ida_tryblks.tryblks_t_grow(self, *args)

    def capacity(self, *args) -> "size_t":
        r"""
        capacity(self) -> size_t
        """
        return _ida_tryblks.tryblks_t_capacity(self, *args)

    def reserve(self, *args) -> "void":
        r"""
        reserve(self, cnt)

        @param cnt: size_t
        """
        return _ida_tryblks.tryblks_t_reserve(self, *args)

    def truncate(self, *args) -> "void":
        r"""
        truncate(self)
        """
        return _ida_tryblks.tryblks_t_truncate(self, *args)

    def swap(self, *args) -> "void":
        r"""
        swap(self, r)

        @param r: qvector< tryblk_t > &
        """
        return _ida_tryblks.tryblks_t_swap(self, *args)

    def extract(self, *args) -> "tryblk_t *":
        r"""
        extract(self) -> tryblk_t
        """
        return _ida_tryblks.tryblks_t_extract(self, *args)

    def inject(self, *args) -> "void":
        r"""
        inject(self, s, len)

        @param s: tryblk_t *
        @param len: size_t
        """
        return _ida_tryblks.tryblks_t_inject(self, *args)

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: qvector< tryblk_t > const &
        """
        return _ida_tryblks.tryblks_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: qvector< tryblk_t > const &
        """
        return _ida_tryblks.tryblks_t___ne__(self, *args)

    def begin(self, *args) -> "qvector< tryblk_t >::const_iterator":
        r"""
        begin(self) -> tryblk_t
        """
        return _ida_tryblks.tryblks_t_begin(self, *args)

    def end(self, *args) -> "qvector< tryblk_t >::const_iterator":
        r"""
        end(self) -> tryblk_t
        """
        return _ida_tryblks.tryblks_t_end(self, *args)

    def insert(self, *args) -> "qvector< tryblk_t >::iterator":
        r"""
        insert(self, it, x) -> tryblk_t

        @param it: qvector< tryblk_t >::iterator
        @param x: tryblk_t const &
        """
        return _ida_tryblks.tryblks_t_insert(self, *args)

    def erase(self, *args) -> "qvector< tryblk_t >::iterator":
        r"""
        erase(self, it) -> tryblk_t

        @param it: qvector< tryblk_t >::iterator

        erase(self, first, last) -> tryblk_t

        @param first: qvector< tryblk_t >::iterator
        @param last: qvector< tryblk_t >::iterator
        """
        return _ida_tryblks.tryblks_t_erase(self, *args)

    def find(self, *args) -> "qvector< tryblk_t >::const_iterator":
        r"""
        find(self, x) -> tryblk_t

        @param x: tryblk_t const &

        """
        return _ida_tryblks.tryblks_t_find(self, *args)

    def has(self, *args) -> "bool":
        r"""
        has(self, x) -> bool

        @param x: tryblk_t const &
        """
        return _ida_tryblks.tryblks_t_has(self, *args)

    def add_unique(self, *args) -> "bool":
        r"""
        add_unique(self, x) -> bool

        @param x: tryblk_t const &
        """
        return _ida_tryblks.tryblks_t_add_unique(self, *args)

    def _del(self, *args) -> "bool":
        r"""
        _del(self, x) -> bool

        Parameters
        ----------
        x: tryblk_t const &

        """
        return _ida_tryblks.tryblks_t__del(self, *args)

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_tryblks.tryblks_t___len__(self, *args)

    def __getitem__(self, *args) -> "tryblk_t const &":
        r"""
        __getitem__(self, i) -> tryblk_t

        @param i: size_t
        """
        return _ida_tryblks.tryblks_t___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: tryblk_t const &
        """
        return _ida_tryblks.tryblks_t___setitem__(self, *args)

    front = ida_idaapi._qvector_front
    back = ida_idaapi._qvector_back
    __iter__ = ida_idaapi._bounded_getitem_iterator


# Register tryblks_t in _ida_tryblks:
_ida_tryblks.tryblks_t_swigregister(tryblks_t)

class catchvec_t(object):
    r"""
    Proxy of C++ qvector< catch_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> catchvec_t
        __init__(self, x) -> catchvec_t

        @param x: qvector< catch_t > const &
        """
        _ida_tryblks.catchvec_t_swiginit(self, _ida_tryblks.new_catchvec_t(*args))
    __swig_destroy__ = _ida_tryblks.delete_catchvec_t

    def push_back(self, *args) -> "catch_t &":
        r"""
        push_back(self, x)

        @param x: catch_t const &

        push_back(self) -> catch_t
        """
        return _ida_tryblks.catchvec_t_push_back(self, *args)

    def pop_back(self, *args) -> "void":
        r"""
        pop_back(self)
        """
        return _ida_tryblks.catchvec_t_pop_back(self, *args)

    def size(self, *args) -> "size_t":
        r"""
        size(self) -> size_t
        """
        return _ida_tryblks.catchvec_t_size(self, *args)

    def empty(self, *args) -> "bool":
        r"""
        empty(self) -> bool
        """
        return _ida_tryblks.catchvec_t_empty(self, *args)

    def at(self, *args) -> "catch_t const &":
        r"""
        at(self, _idx) -> catch_t

        @param _idx: size_t
        """
        return _ida_tryblks.catchvec_t_at(self, *args)

    def qclear(self, *args) -> "void":
        r"""
        qclear(self)
        """
        return _ida_tryblks.catchvec_t_qclear(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_tryblks.catchvec_t_clear(self, *args)

    def resize(self, *args) -> "void":
        r"""
        resize(self, _newsize, x)

        @param _newsize: size_t
        @param x: catch_t const &

        resize(self, _newsize)

        @param _newsize: size_t
        """
        return _ida_tryblks.catchvec_t_resize(self, *args)

    def grow(self, *args) -> "void":
        r"""
        grow(self, x=catch_t())

        @param x: catch_t const &
        """
        return _ida_tryblks.catchvec_t_grow(self, *args)

    def capacity(self, *args) -> "size_t":
        r"""
        capacity(self) -> size_t
        """
        return _ida_tryblks.catchvec_t_capacity(self, *args)

    def reserve(self, *args) -> "void":
        r"""
        reserve(self, cnt)

        @param cnt: size_t
        """
        return _ida_tryblks.catchvec_t_reserve(self, *args)

    def truncate(self, *args) -> "void":
        r"""
        truncate(self)
        """
        return _ida_tryblks.catchvec_t_truncate(self, *args)

    def swap(self, *args) -> "void":
        r"""
        swap(self, r)

        @param r: qvector< catch_t > &
        """
        return _ida_tryblks.catchvec_t_swap(self, *args)

    def extract(self, *args) -> "catch_t *":
        r"""
        extract(self) -> catch_t
        """
        return _ida_tryblks.catchvec_t_extract(self, *args)

    def inject(self, *args) -> "void":
        r"""
        inject(self, s, len)

        @param s: catch_t *
        @param len: size_t
        """
        return _ida_tryblks.catchvec_t_inject(self, *args)

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: qvector< catch_t > const &
        """
        return _ida_tryblks.catchvec_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: qvector< catch_t > const &
        """
        return _ida_tryblks.catchvec_t___ne__(self, *args)

    def begin(self, *args) -> "qvector< catch_t >::const_iterator":
        r"""
        begin(self) -> catch_t
        """
        return _ida_tryblks.catchvec_t_begin(self, *args)

    def end(self, *args) -> "qvector< catch_t >::const_iterator":
        r"""
        end(self) -> catch_t
        """
        return _ida_tryblks.catchvec_t_end(self, *args)

    def insert(self, *args) -> "qvector< catch_t >::iterator":
        r"""
        insert(self, it, x) -> catch_t

        @param it: qvector< catch_t >::iterator
        @param x: catch_t const &
        """
        return _ida_tryblks.catchvec_t_insert(self, *args)

    def erase(self, *args) -> "qvector< catch_t >::iterator":
        r"""
        erase(self, it) -> catch_t

        @param it: qvector< catch_t >::iterator

        erase(self, first, last) -> catch_t

        @param first: qvector< catch_t >::iterator
        @param last: qvector< catch_t >::iterator
        """
        return _ida_tryblks.catchvec_t_erase(self, *args)

    def find(self, *args) -> "qvector< catch_t >::const_iterator":
        r"""
        find(self, x) -> catch_t

        @param x: catch_t const &

        """
        return _ida_tryblks.catchvec_t_find(self, *args)

    def has(self, *args) -> "bool":
        r"""
        has(self, x) -> bool

        @param x: catch_t const &
        """
        return _ida_tryblks.catchvec_t_has(self, *args)

    def add_unique(self, *args) -> "bool":
        r"""
        add_unique(self, x) -> bool

        @param x: catch_t const &
        """
        return _ida_tryblks.catchvec_t_add_unique(self, *args)

    def _del(self, *args) -> "bool":
        r"""
        _del(self, x) -> bool

        Parameters
        ----------
        x: catch_t const &

        """
        return _ida_tryblks.catchvec_t__del(self, *args)

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_tryblks.catchvec_t___len__(self, *args)

    def __getitem__(self, *args) -> "catch_t const &":
        r"""
        __getitem__(self, i) -> catch_t

        @param i: size_t
        """
        return _ida_tryblks.catchvec_t___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: catch_t const &
        """
        return _ida_tryblks.catchvec_t___setitem__(self, *args)

    front = ida_idaapi._qvector_front
    back = ida_idaapi._qvector_back
    __iter__ = ida_idaapi._bounded_getitem_iterator


# Register catchvec_t in _ida_tryblks:
_ida_tryblks.catchvec_t_swigregister(catchvec_t)

class try_handler_t(ida_range.rangevec_t):
    r"""
    Proxy of C++ try_handler_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    disp = property(_ida_tryblks.try_handler_t_disp_get, _ida_tryblks.try_handler_t_disp_set, doc=r"""disp""")
    fpreg = property(_ida_tryblks.try_handler_t_fpreg_get, _ida_tryblks.try_handler_t_fpreg_set, doc=r"""fpreg""")

    def __init__(self, *args):
        r"""
        __init__(self) -> try_handler_t
        """
        _ida_tryblks.try_handler_t_swiginit(self, _ida_tryblks.new_try_handler_t(*args))

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_tryblks.try_handler_t_clear(self, *args)
    __swig_destroy__ = _ida_tryblks.delete_try_handler_t

# Register try_handler_t in _ida_tryblks:
_ida_tryblks.try_handler_t_swigregister(try_handler_t)

class seh_t(try_handler_t):
    r"""
    Proxy of C++ seh_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    filter = property(_ida_tryblks.seh_t_filter_get, _ida_tryblks.seh_t_filter_set, doc=r"""filter""")
    seh_code = property(_ida_tryblks.seh_t_seh_code_get, _ida_tryblks.seh_t_seh_code_set, doc=r"""seh_code""")

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_tryblks.seh_t_clear(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> seh_t
        """
        _ida_tryblks.seh_t_swiginit(self, _ida_tryblks.new_seh_t(*args))
    __swig_destroy__ = _ida_tryblks.delete_seh_t

# Register seh_t in _ida_tryblks:
_ida_tryblks.seh_t_swigregister(seh_t)

class catch_t(try_handler_t):
    r"""
    Proxy of C++ catch_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    obj = property(_ida_tryblks.catch_t_obj_get, _ida_tryblks.catch_t_obj_set, doc=r"""obj""")
    type_id = property(_ida_tryblks.catch_t_type_id_get, _ida_tryblks.catch_t_type_id_set, doc=r"""type_id""")

    def __init__(self, *args):
        r"""
        __init__(self) -> catch_t
        """
        _ida_tryblks.catch_t_swiginit(self, _ida_tryblks.new_catch_t(*args))
    __swig_destroy__ = _ida_tryblks.delete_catch_t

# Register catch_t in _ida_tryblks:
_ida_tryblks.catch_t_swigregister(catch_t)

class tryblk_t(ida_range.rangevec_t):
    r"""
    Proxy of C++ tryblk_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    level = property(_ida_tryblks.tryblk_t_level_get, _ida_tryblks.tryblk_t_level_set, doc=r"""level""")

    def cpp(self, *args) -> "catchvec_t &":
        r"""
        cpp(self) -> catchvec_t
        """
        return _ida_tryblks.tryblk_t_cpp(self, *args)

    def seh(self, *args) -> "seh_t &":
        r"""
        seh(self) -> seh_t
        """
        return _ida_tryblks.tryblk_t_seh(self, *args)
    __swig_destroy__ = _ida_tryblks.delete_tryblk_t

    def __init__(self, *args):
        r"""
        __init__(self) -> tryblk_t
        __init__(self, r) -> tryblk_t

        @param r: tryblk_t const &
        """
        _ida_tryblks.tryblk_t_swiginit(self, _ida_tryblks.new_tryblk_t(*args))

    def get_kind(self, *args) -> "uchar":
        r"""
        get_kind(self) -> uchar
        """
        return _ida_tryblks.tryblk_t_get_kind(self, *args)

    def empty(self, *args) -> "bool":
        r"""
        empty(self) -> bool
        """
        return _ida_tryblks.tryblk_t_empty(self, *args)

    def is_seh(self, *args) -> "bool":
        r"""
        is_seh(self) -> bool
        """
        return _ida_tryblks.tryblk_t_is_seh(self, *args)

    def is_cpp(self, *args) -> "bool":
        r"""
        is_cpp(self) -> bool
        """
        return _ida_tryblks.tryblk_t_is_cpp(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_tryblks.tryblk_t_clear(self, *args)

    def set_seh(self, *args) -> "seh_t &":
        r"""
        set_seh(self) -> seh_t
        """
        return _ida_tryblks.tryblk_t_set_seh(self, *args)

    def set_cpp(self, *args) -> "catchvec_t &":
        r"""
        set_cpp(self) -> catchvec_t
        """
        return _ida_tryblks.tryblk_t_set_cpp(self, *args)

# Register tryblk_t in _ida_tryblks:
_ida_tryblks.tryblk_t_swigregister(tryblk_t)


def get_tryblks(*args) -> "size_t":
    r"""
    get_tryblks(tbv, range) -> size_t
    Retrieve try block information from the specified address range. Try blocks are
    sorted by starting address and their nest levels calculated.

    @param tbv: (C++: tryblks_t *) output buffer; may be nullptr
    @param range: (C++: const range_t &) address range to change
    @return: number of found try blocks
    """
    return _ida_tryblks.get_tryblks(*args)

def del_tryblks(*args) -> "void":
    r"""
    del_tryblks(range)
    Delete try block information in the specified range.

    @param range: (C++: const range_t &) the range to be cleared
    """
    return _ida_tryblks.del_tryblks(*args)

def add_tryblk(*args) -> "int":
    r"""
    add_tryblk(tb) -> int
    Add one try block information.

    @param tb: (C++: const tryblk_t &) try block to add.
    @return: error code; 0 means good
    """
    return _ida_tryblks.add_tryblk(*args)
TBERR_OK = _ida_tryblks.TBERR_OK
r"""
ok
"""

TBERR_START = _ida_tryblks.TBERR_START
r"""
bad start address
"""

TBERR_END = _ida_tryblks.TBERR_END
r"""
bad end address
"""

TBERR_ORDER = _ida_tryblks.TBERR_ORDER
r"""
bad address order
"""

TBERR_EMPTY = _ida_tryblks.TBERR_EMPTY
r"""
empty try block
"""

TBERR_KIND = _ida_tryblks.TBERR_KIND
r"""
illegal try block kind
"""

TBERR_NO_CATCHES = _ida_tryblks.TBERR_NO_CATCHES
r"""
no catch blocks at all
"""

TBERR_INTERSECT = _ida_tryblks.TBERR_INTERSECT
r"""
range would intersect inner tryblk
"""


def find_syseh(*args) -> "ea_t":
    r"""
    find_syseh(ea) -> ea_t
    Find the start address of the system eh region including the argument.

    @param ea: (C++: ea_t) search address
    @return: start address of surrounding tryblk, otherwise BADADDR
    """
    return _ida_tryblks.find_syseh(*args)
TBEA_TRY = _ida_tryblks.TBEA_TRY

TBEA_CATCH = _ida_tryblks.TBEA_CATCH

TBEA_SEHTRY = _ida_tryblks.TBEA_SEHTRY

TBEA_SEHLPAD = _ida_tryblks.TBEA_SEHLPAD

TBEA_SEHFILT = _ida_tryblks.TBEA_SEHFILT

TBEA_ANY = _ida_tryblks.TBEA_ANY

TBEA_FALLTHRU = _ida_tryblks.TBEA_FALLTHRU


def is_ea_tryblks(*args) -> "bool":
    r"""
    is_ea_tryblks(ea, flags) -> bool
    Check if the given address ea is part of tryblks description.

    @param ea: (C++: ea_t) address to check
    @param flags: (C++: uint32) combination of flags for is_ea_tryblks()
    """
    return _ida_tryblks.is_ea_tryblks(*args)






"""
summary: an `ida_idp.IDP_Hooks.assembly` implementation

description:
  We add support for assembling the following pseudo instructions:

  * "zero eax" -> xor eax, eax
  * "nothing" -> nop
"""

import ida_idp
import idautils

#--------------------------------------------------------------------------
class assemble_idp_hook_t(ida_idp.IDP_Hooks):
    def ev_assemble(self, ea, cs, ip, use32, line):
        line = line.strip()
        if line == "zero eax":
            return b"\x33\xC0"
        elif line == "nothing":
            # Decode current instruction to figure out its size
            cmd = idautils.DecodeInstruction(ea)
            if cmd:
                # NOP all the instruction bytes
                return b"\x90" * cmd.size
        return None


#---------------------------------------------------------------------
# Remove an existing hook on second run
try:
    idp_hook_stat = "un"
    print("IDP hook: checking for hook...")
    idphook
    print("IDP hook: unhooking....")
    idphook.unhook()
    del idphook
except:
    print("IDP hook: not installed, installing now....")
    idp_hook_stat = ""
    idphook = assemble_idp_hook_t()
    idphook.hook()

print("IDP hook %sinstalled. Run the script again to %sinstall" % (idp_hook_stat, idp_hook_stat))



"""
summary: list all functions (and xrefs) in segment

description:
  List all the functions in the current segment, as well as
  all the cross-references to them.

  Contrary to @list_segment_functions, this uses the somewhat
  higher-level `idautils` module.

keywords: xrefs

see_also: list_segment_functions
"""

#
# Reference Lister
#
# List all functions and all references to them in the current section.
#
# Implemented with the idautils module
#

import ida_kernwin
import ida_idaapi
import ida_segment
import ida_funcs

import idautils

def main():
    # Get current ea
    ea = ida_kernwin.get_screen_ea()
    if ea == ida_idaapi.BADADDR:
        print("Could not get get_screen_ea()")
        return

    seg = ida_segment.getseg(ea)
    if seg:
        # Loop from start to end in the current segment
        for funcea in idautils.Functions(seg.start_ea, seg.end_ea):
            print("Function %s at 0x%x" % (ida_funcs.get_func_name(funcea), funcea))

            # Find all code references to funcea
            for ref in idautils.CodeRefsTo(funcea, 1):
                print("  called from %s(0x%x)" % (ida_funcs.get_func_name(ref), ref))
    else:
        print("Please position the cursor within a segment")

if __name__=='__main__':
    main()




from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_idc
else:
    import _ida_idc

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_idc.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi


def mark_position(*args) -> "void":
    r"""
    mark_position(ea, lnnum, x, y, slot, comment)

    @param ea: ea_t
    @param lnnum: int
    @param x: short
    @param y: short
    @param slot: int32
    @param comment: char const *
    """
    return _ida_idc.mark_position(*args)

def get_marked_pos(*args) -> "ea_t":
    r"""
    get_marked_pos(slot) -> ea_t

    @param slot: int32
    """
    return _ida_idc.get_marked_pos(*args)

def get_mark_comment(*args) -> "PyObject *":
    r"""
    get_mark_comment(slot) -> PyObject *

    @param slot: int32
    """
    return _ida_idc.get_mark_comment(*args)

#<pycode(py_idc)>
#</pycode(py_idc)>







"""
IEEE floating point functions."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_ieee
else:
    import _ida_ieee

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_ieee.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

class fpvalue_shorts_array_t(object):
    r"""
    Proxy of C++ wrapped_array_t< uint16,FPVAL_NWORDS > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data = property(_ida_ieee.fpvalue_shorts_array_t_data_get, doc=r"""data""")

    def __init__(self, *args):
        r"""
        __init__(self, data) -> fpvalue_shorts_array_t

        @param data: unsigned short (&)[FPVAL_NWORDS]
        """
        _ida_ieee.fpvalue_shorts_array_t_swiginit(self, _ida_ieee.new_fpvalue_shorts_array_t(*args))

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_ieee.fpvalue_shorts_array_t___len__(self, *args)

    def __getitem__(self, *args) -> "unsigned short const &":
        r"""
        __getitem__(self, i) -> unsigned short const &

        @param i: size_t
        """
        return _ida_ieee.fpvalue_shorts_array_t___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: unsigned short const &
        """
        return _ida_ieee.fpvalue_shorts_array_t___setitem__(self, *args)

    def _get_bytes(self, *args) -> "bytevec_t":
        r"""_get_bytes(self) -> bytevec_t"""
        return _ida_ieee.fpvalue_shorts_array_t__get_bytes(self, *args)

    def _set_bytes(self, *args) -> "void":
        r"""
        _set_bytes(self, bts)

        Parameters
        ----------
        bts: bytevec_t const &

        """
        return _ida_ieee.fpvalue_shorts_array_t__set_bytes(self, *args)

    __iter__ = ida_idaapi._bounded_getitem_iterator
    bytes = property(_get_bytes, _set_bytes)

    __swig_destroy__ = _ida_ieee.delete_fpvalue_shorts_array_t

# Register fpvalue_shorts_array_t in _ida_ieee:
_ida_ieee.fpvalue_shorts_array_t_swigregister(fpvalue_shorts_array_t)

FPVAL_NWORDS = _ida_ieee.FPVAL_NWORDS

FPV_BADARG = _ida_ieee.FPV_BADARG
r"""
wrong value of max_exp
"""

FPV_NORM = _ida_ieee.FPV_NORM
r"""
regular value
"""

FPV_NAN = _ida_ieee.FPV_NAN
r"""
NaN.
"""

FPV_PINF = _ida_ieee.FPV_PINF
r"""
positive infinity
"""

FPV_NINF = _ida_ieee.FPV_NINF
r"""
negative infinity
"""

REAL_ERROR_OK = _ida_ieee.REAL_ERROR_OK
r"""
no error
"""

REAL_ERROR_FORMAT = _ida_ieee.REAL_ERROR_FORMAT

REAL_ERROR_RANGE = _ida_ieee.REAL_ERROR_RANGE

REAL_ERROR_BADDATA = _ida_ieee.REAL_ERROR_BADDATA

REAL_ERROR_FPOVER = _ida_ieee.REAL_ERROR_FPOVER
r"""
floating overflow or underflow
"""

REAL_ERROR_BADSTR = _ida_ieee.REAL_ERROR_BADSTR

REAL_ERROR_ZERODIV = _ida_ieee.REAL_ERROR_ZERODIV

REAL_ERROR_INTOVER = _ida_ieee.REAL_ERROR_INTOVER
r"""
eetol*: integer overflow
"""

class fpvalue_t(object):
    r"""
    Proxy of C++ fpvalue_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    w = property(_ida_ieee.fpvalue_t_w_get, _ida_ieee.fpvalue_t_w_set, doc=r"""w""")

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_ieee.fpvalue_t_clear(self, *args)

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___ne__(self, *args)

    def __lt__(self, *args) -> "bool":
        r"""
        __lt__(self, r) -> bool

        @param r: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___lt__(self, *args)

    def __gt__(self, *args) -> "bool":
        r"""
        __gt__(self, r) -> bool

        @param r: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___gt__(self, *args)

    def __le__(self, *args) -> "bool":
        r"""
        __le__(self, r) -> bool

        @param r: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___le__(self, *args)

    def __ge__(self, *args) -> "bool":
        r"""
        __ge__(self, r) -> bool

        @param r: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___ge__(self, *args)

    def compare(self, *args) -> "int":
        r"""
        compare(self, r) -> int

        @param r: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t_compare(self, *args)

    def from_10bytes(self, *args) -> "fpvalue_error_t":
        r"""
        from_10bytes(self, fpval) -> fpvalue_error_t
        Conversions for 10-byte floating point values.

        @param fpval: (C++: const void *) void const *
        """
        return _ida_ieee.fpvalue_t_from_10bytes(self, *args)

    def to_10bytes(self, *args) -> "fpvalue_error_t":
        r"""
        to_10bytes(self, fpval) -> fpvalue_error_t

        @param fpval: void *
        """
        return _ida_ieee.fpvalue_t_to_10bytes(self, *args)

    def from_12bytes(self, *args) -> "fpvalue_error_t":
        r"""
        from_12bytes(self, fpval) -> fpvalue_error_t
        Conversions for 12-byte floating point values.

        @param fpval: (C++: const void *) void const *
        """
        return _ida_ieee.fpvalue_t_from_12bytes(self, *args)

    def to_12bytes(self, *args) -> "fpvalue_error_t":
        r"""
        to_12bytes(self, fpval) -> fpvalue_error_t

        @param fpval: void *
        """
        return _ida_ieee.fpvalue_t_to_12bytes(self, *args)

    def to_str(self, *args) -> "void":
        r"""
        to_str(self, mode)
        Convert IEEE to string.

        @param mode: (C++: uint) broken down into:
        * low byte: number of digits after '.'
        * second byte: FPNUM_LENGTH
        * third byte: FPNUM_DIGITS
        """
        return _ida_ieee.fpvalue_t_to_str(self, *args)

    def from_sval(self, *args) -> "void":
        r"""
        from_sval(self, x)
        Convert integer to IEEE.

        @param x: (C++: sval_t)
        """
        return _ida_ieee.fpvalue_t_from_sval(self, *args)

    def from_int64(self, *args) -> "void":
        r"""
        from_int64(self, x)

        @param x: int64
        """
        return _ida_ieee.fpvalue_t_from_int64(self, *args)

    def from_uint64(self, *args) -> "void":
        r"""
        from_uint64(self, x)

        @param x: uint64
        """
        return _ida_ieee.fpvalue_t_from_uint64(self, *args)

    def to_sval(self, *args) -> "fpvalue_error_t":
        r"""
        to_sval(self, round=False) -> fpvalue_error_t
        Convert IEEE to integer (+-0.5 if round)

        @param round: (C++: bool)
        """
        return _ida_ieee.fpvalue_t_to_sval(self, *args)

    def to_int64(self, *args) -> "fpvalue_error_t":
        r"""
        to_int64(self, round=False) -> fpvalue_error_t

        @param round: bool
        """
        return _ida_ieee.fpvalue_t_to_int64(self, *args)

    def to_uint64(self, *args) -> "fpvalue_error_t":
        r"""
        to_uint64(self, round=False) -> fpvalue_error_t

        @param round: bool
        """
        return _ida_ieee.fpvalue_t_to_uint64(self, *args)

    def fadd(self, *args) -> "fpvalue_error_t":
        r"""
        fadd(self, y) -> fpvalue_error_t
        Arithmetic operations.

        @param y: (C++: const fpvalue_t &) fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t_fadd(self, *args)

    def fsub(self, *args) -> "fpvalue_error_t":
        r"""
        fsub(self, y) -> fpvalue_error_t

        @param y: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t_fsub(self, *args)

    def fmul(self, *args) -> "fpvalue_error_t":
        r"""
        fmul(self, y) -> fpvalue_error_t

        @param y: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t_fmul(self, *args)

    def fdiv(self, *args) -> "fpvalue_error_t":
        r"""
        fdiv(self, y) -> fpvalue_error_t

        @param y: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t_fdiv(self, *args)

    def mul_pow2(self, *args) -> "fpvalue_error_t":
        r"""
        mul_pow2(self, power_of_2) -> fpvalue_error_t
        Multiply by a power of 2.

        @param power_of_2: (C++: int32)
        """
        return _ida_ieee.fpvalue_t_mul_pow2(self, *args)

    def eabs(self, *args) -> "void":
        r"""
        eabs(self)
        Calculate absolute value.
        """
        return _ida_ieee.fpvalue_t_eabs(self, *args)

    def is_negative(self, *args) -> "bool":
        r"""
        is_negative(self) -> bool
        Is negative value?
        """
        return _ida_ieee.fpvalue_t_is_negative(self, *args)

    def negate(self, *args) -> "void":
        r"""
        negate(self)
        Negate.
        """
        return _ida_ieee.fpvalue_t_negate(self, *args)

    def get_kind(self, *args) -> "fpvalue_kind_t":
        r"""
        get_kind(self) -> fpvalue_kind_t
        Get value kind.
        """
        return _ida_ieee.fpvalue_t_get_kind(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> fpvalue_t
        __init__(self, _in) -> fpvalue_t

        @param in: bytevec12_t const &
        """
        _ida_ieee.fpvalue_t_swiginit(self, _ida_ieee.new_fpvalue_t(*args))

    def _get_bytes(self, *args) -> "void":
        r"""_get_bytes(self)"""
        return _ida_ieee.fpvalue_t__get_bytes(self, *args)

    def _set_bytes(self, *args) -> "void":
        r"""
        _set_bytes(self, _in)

        Parameters
        ----------
        in: bytevec12_t const &

        """
        return _ida_ieee.fpvalue_t__set_bytes(self, *args)

    def _get_10bytes(self, *args) -> "void":
        r"""_get_10bytes(self)"""
        return _ida_ieee.fpvalue_t__get_10bytes(self, *args)

    def _set_10bytes(self, *args) -> "void":
        r"""
        _set_10bytes(self, _in)

        Parameters
        ----------
        in: bytevec10_t const &

        """
        return _ida_ieee.fpvalue_t__set_10bytes(self, *args)

    def _get_float(self, *args) -> "double":
        r"""_get_float(self) -> double"""
        return _ida_ieee.fpvalue_t__get_float(self, *args)

    def _set_float(self, *args) -> "void":
        r"""
        _set_float(self, v)

        Parameters
        ----------
        v: double

        """
        return _ida_ieee.fpvalue_t__set_float(self, *args)

    def __str__(self, *args) -> "qstring":
        r"""
        __str__(self) -> qstring
        """
        return _ida_ieee.fpvalue_t___str__(self, *args)

    def _get_shorts(self, *args) -> "wrapped_array_t< uint16,FPVAL_NWORDS >":
        r"""_get_shorts(self) -> fpvalue_shorts_array_t"""
        return _ida_ieee.fpvalue_t__get_shorts(self, *args)

    def from_str(self, *args) -> "fpvalue_error_t":
        r"""
        from_str(self, p) -> fpvalue_error_t
        Convert string to IEEE.

        @param p_str: (C++: const char **) pointer to pointer to string. it will advanced.
        """
        return _ida_ieee.fpvalue_t_from_str(self, *args)

    def assign(self, *args) -> "void":
        r"""
        assign(self, r)

        @param r: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t_assign(self, *args)

    bytes = property(_get_bytes, _set_bytes)
    _10bytes = property(_get_10bytes, _set_10bytes)
    shorts = property(_get_shorts)
    float = property(_get_float, _set_float)
    sval = property(lambda self: self.to_sval(), lambda self, v: self.from_sval(v))
    int64 = property(lambda self: self.to_int64(), lambda self, v: self.from_int64(v))
    uint64 = property(lambda self: self.to_uint64(), lambda self, v: self.from_uint64(v))

    def __iter__(self):
        shorts = self.shorts
        for one in shorts:
            yield one

    def __getitem__(self, i):
        return self.shorts[i]

    def __setitem__(self, i, v):
        self.shorts[i] = v


    def __add__(self, *args) -> "fpvalue_t":
        r"""
        __add__(self, o) -> fpvalue_t

        @param o: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___add__(self, *args)

    def __sub__(self, *args) -> "fpvalue_t":
        r"""
        __sub__(self, o) -> fpvalue_t

        @param o: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___sub__(self, *args)

    def __mul__(self, *args) -> "fpvalue_t":
        r"""
        __mul__(self, o) -> fpvalue_t

        @param o: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___mul__(self, *args)

    def __truediv__(self, *args) -> "fpvalue_t":
        r"""
        __truediv__(self, o) -> fpvalue_t

        @param o: fpvalue_t const &
        """
        return _ida_ieee.fpvalue_t___truediv__(self, *args)
    __swig_destroy__ = _ida_ieee.delete_fpvalue_t

# Register fpvalue_t in _ida_ieee:
_ida_ieee.fpvalue_t_swigregister(fpvalue_t)
cvar = _ida_ieee.cvar
MAXEXP_FLOAT = cvar.MAXEXP_FLOAT
MAXEXP_DOUBLE = cvar.MAXEXP_DOUBLE
MAXEXP_LNGDBL = cvar.MAXEXP_LNGDBL

IEEE_EXONE = _ida_ieee.IEEE_EXONE
r"""
The exponent of 1.0.
"""

E_SPECIAL_EXP = _ida_ieee.E_SPECIAL_EXP
r"""
Exponent in fpvalue_t for NaN and Inf.
"""

IEEE_NI = _ida_ieee.IEEE_NI

IEEE_E = _ida_ieee.IEEE_E

IEEE_M = _ida_ieee.IEEE_M


def ecleaz(*args) -> "void":
    r"""
    ecleaz(x)

    @param x: unsigned short [(6+3)]
    """
    return _ida_ieee.ecleaz(*args)

#<pycode(py_ieee)>
EZERO = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
EONE = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x80\xFF\x3F"
ETWO = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x80\x00\x40"
#</pycode(py_ieee)>







"""
summary: using timers for delayed execution

description:
  Register (possibly repeating) timers.
"""

import ida_kernwin

# -------------------------------------------------------------------------
class timercallback_t(object):
    def __init__(self):
        self.interval = 1000
        self.obj = ida_kernwin.register_timer(self.interval, self)
        if self.obj is None:
            raise RuntimeError("Failed to register timer")
        self.times = 5

    def __call__(self):
        print("Timer invoked. %d time(s) left" % self.times)
        self.times -= 1
        # Unregister the timer when the counter reaches zero
        return -1 if self.times == 0 else self.interval

    def __del__(self):
        print("Timer object disposed %s" % self)


# -------------------------------------------------------------------------
def main():
    try:
        t = timercallback_t()
        # No need to unregister the timer.
        # It will unregister itself in the callback when it returns -1
    except Exception as e:
        print("Error: %s" % e)


# -------------------------------------------------------------------------
if __name__ == '__main__':
    main()



import sys
import requests
import idaapi
from packaging import version

from yaraforge.version import __version__
from yaraforge.utils.logger import get_global_logger

logger = get_global_logger()


def check_for_updates(force_check=False):
    try:
        # 检查网络连接状态
        if not is_network_available():
            if force_check:
                print("[YaraForge] No network connection. Update check skipped.")
            return

        print(f"[YaraForge] Current version: {__version__}")
        logger.info(f"Current version: {__version__}")

        # 从 PyPI 获取最新版本号
        pypi_url = "https://pypi.org/pypi/yaraforge/json"
        response = requests.get(pypi_url)
        response.raise_for_status()
        latest_version_str = response.json()["info"]["version"]

        current_version = version.parse(__version__)
        latest_version = version.parse(latest_version_str)

        if latest_version > current_version:
            print(f"[YaraForge] New version available: {latest_version}")
            logger.info(f"New version available: {latest_version}")
            if idaapi.ask_yn(idaapi.ASKBTN_YES, f"A new version of YaraForge ({latest_version}) is available. Do you "
                                                f"want to update now?") == idaapi.ASKBTN_YES:
                perform_update()
            else:
                print("[YaraForge] You can update later by running the 'yf-update' command in the Python Console.")
        else:
            print("[YaraForge] No updates available.")
            logger.info("No updates available.")
    except requests.exceptions.RequestException as e:
        print(f"[YaraForge] Update check failed: {e}")
        logger.error(f"Update check failed: {e}")


def perform_update():
    try:
        # 直接使用 pip 执行更新
        import subprocess
        subprocess.check_call(["pip", "install", "--upgrade", "yaraforge"])
        print("[YaraForge] Update completed. Please restart IDA Pro for the changes to take effect.")
        logger.info("Update completed. Please restart IDA Pro for the changes to take effect.")
    except subprocess.CalledProcessError as e:
        print("[YaraForge] Update failed. You can still use the current version of YaraForge.")
        print("Please check the IDAPython console for more information.")
        print("You can try updating YaraForge manually by running the following command:")
        print("    pip install --upgrade yaraforge")
        print("Make sure to use the correct Python interpreter associated with your IDA Pro installation.")
        print("If the issue persists, please contact the author or open an issue on GitHub:")
        print("- Author: zhaoxinnZ (zhaoxinzhang0429@gmail.com)")
        print("- GitHub Issues: https://github.com/zhaoxinnZ/YaraForge/issues")
        logger.error(f"Update failed: {e}")
    except Exception as e:
        print(f"[YaraForge] Update failed: {e}")
        print("You can still use the current version of YaraForge.")
        print("Please check the IDAPython console for more information.")
        print("If the issue persists, please contact the author or open an issue on GitHub:")
        print("- Author: zhaoxinnZ (zhaoxinzhang0429@gmail.com)")
        print("- GitHub Issues: https://github.com/zhaoxinnZ/YaraForge/issues")
        logger.error(f"Update failed: {e}")


def is_network_available():
    import socket
    try:
        socket.create_connection(("www.google.com", 80))
        return True
    except OSError:
        return False


"""
summary: Record and replay changes in function prototypes

description:
    This is a sample script, that will record (in memory) all changes in
    functions prototypes, in order to re-apply them later.

    To use this script:
     - open an IDB (say, "test.idb")
     - modify some functions prototypes (e.g., by triggering the 'Y'
       shortcut when the cursor is placed on the first address of a
       function)
     - reload that IDB, *without saving it first*
     - call rpc.replay(), to re-apply the modifications.

    Note: 'ti_changed' is also called for changes to the function
    frames, but we'll only record function prototypes changes.
"""
import ida_idp
import ida_funcs
import ida_typeinf

class replay_prototypes_changes_t(ida_idp.IDB_Hooks):
    def __init__(self):
        ida_idp.IDB_Hooks.__init__(self)
        # we'll store tuples (ea, typ, fields). We cannot store
        # tinfo_t instances in there, because tinfo_t's are only
        # valid while the IDB is opened.
        # Since the very purpose of this example is to re-apply
        # types after the IDB has been closed & re-opened, we
        # must therefore keep the serialized version only.
        self.memo = []
        self.replaying = False

    def _deser(self, typ, fields):
        tif = ida_typeinf.tinfo_t()
        if not tif.deserialize(ida_typeinf.get_idati(), typ, fields):
            tif = None
        return tif

    def ti_changed(self, ea, typ, fields):
        if not self.replaying:
            pfn = ida_funcs.get_func(ea)
            if pfn and pfn.start_ea == ea:
                self.memo.append((ea, typ, fields))

                # de-serialize, just for the sake of printing
                tif = self._deser(typ, fields)
                if tif:
                    print("%x: type changed: %s" % (
                        ea,
                        tif._print(None, ida_typeinf.PRTYPE_1LINE)))

    def replay(self):
        self.replaying = True
        try:
            for ea, typ, fields in self.memo:
                tif = self._deser(typ, fields)
                if tif:
                    print("%x: applying type: %s" % (
                        ea,
                        tif._print(None, ida_typeinf.PRTYPE_1LINE)))
                    # Since that type information was remembered from a change
                    # the user made, we'll re-apply it as a definite type (i.e.,
                    # can't be overriden by IDA's auto-analysis/heuristics.)
                    apply_flags = ida_typeinf.TINFO_DEFINITE
                    if not ida_typeinf.apply_tinfo(ea, tif, apply_flags):
                        print("FAILED")
        finally:
            self.replaying = False

rpc = replay_prototypes_changes_t()
if rpc.hook():
    print("""
Please modify some functions prototypes (press 'Y' when the
cursor is on the function name, or first address), and when
you are done reload this IDB, *WITHOUT* saving it first,
and type 'rpc.replay()'
""")
else:
    print("Couldn't create hooks")



"""
summary: programmatically create & populate a structure

description:
  Usage of the API to create & populate a structure with
  members of different types.

author: Gergely Erdelyi (gergely.erdelyi@d-dome.net)
"""

#---------------------------------------------------------------------
# Structure test
#
# This script demonstrates how to create structures and populate them
# with members of different types.
#---------------------------------------------------------------------

import ida_struct
import ida_idaapi
import ida_bytes
import ida_nalt

import idc

sid = ida_struct.get_struc_id("mystr1")
if sid != -1:
    idc.del_struc(sid)
sid = ida_struct.add_struc(ida_idaapi.BADADDR, "mystr1", 0)
print("%x" % sid)

# Test simple data types
simple_types_data = [
    (ida_bytes.FF_BYTE, 1),
    (ida_bytes.FF_WORD, 2),
    (ida_bytes.FF_DWORD, 4),
    (ida_bytes.FF_QWORD, 8),
    (ida_bytes.FF_TBYTE, 10),
    (ida_bytes.FF_OWORD, 16),
    (ida_bytes.FF_FLOAT, 4),
    (ida_bytes.FF_DOUBLE, 8),
    (ida_bytes.FF_PACKREAL, 10),
]
for i, tpl in enumerate(simple_types_data):
    t, nsize = tpl
    print("t%x:"% ((t|ida_bytes.FF_DATA) & 0xFFFFFFFF),
          idc.add_struc_member(sid, "t%02d"%i, ida_idaapi.BADADDR, (t|ida_bytes.FF_DATA )&0xFFFFFFFF, -1, nsize))

# Test ASCII type
print("ASCII:", idc.add_struc_member(sid, "tascii", -1, ida_bytes.FF_STRLIT|ida_bytes.FF_DATA, ida_nalt.STRTYPE_C, 8))

# Test struc member type
msid = ida_struct.get_struc_id("mystr2")
if msid != -1:
    idc.del_struc(msid)
msid = idc.add_struc(-1, "mystr2", 0)
print(idc.add_struc_member(msid, "member1", -1, (ida_bytes.FF_DWORD|ida_bytes.FF_DATA )&0xFFFFFFFF, -1, 4))
print(idc.add_struc_member(msid, "member2", -1, (ida_bytes.FF_DWORD|ida_bytes.FF_DATA )&0xFFFFFFFF, -1, 4))

msize = ida_struct.get_struc_size(msid)
print("Struct:", idc.add_struc_member(sid, "tstruct", -1, ida_bytes.FF_STRUCT|ida_bytes.FF_DATA, msid, msize))
print("Stroff:", idc.add_struc_member(sid, "tstroff", -1, ida_bytes.stroff_flag()|ida_bytes.FF_DWORD, msid, 4))

# Test offset types
print("Offset:", idc.add_struc_member(sid, "toffset", -1, ida_bytes.off_flag()|ida_bytes.FF_DATA|ida_bytes.FF_DWORD, 0, 4))
print("Offset:", idc.set_member_type(sid, 0, ida_bytes.off_flag()|ida_bytes.FF_DATA|ida_bytes.FF_DWORD, 0, 4))

print("Done")




import ida_dbg
import ida_idaapi
import ida_idd
import ida_kernwin
import ida_typeinf
import ida_name

def log(msg):
    print(">>> %s" % msg)

class appcall_hooks_t(ida_dbg.DBG_Hooks):
    def __init__(self, name_funcs=[]):
        ida_dbg.DBG_Hooks.__init__(self) # important

        for ea, func_name in name_funcs:
            log("Renaming 0x%08x to \"%s\"" % (ea, func_name))
            ida_name.set_name(ea, func_name)

        for func_name, func_proto in [
                ("ref4", "int ref4(int *);"),
                ("ref8", "int ref8(long long int *);"),
        ]:
            log("Setting '%s's prototype" % func_name)
            func_ea = ida_name.get_name_ea(ida_idaapi.BADADDR, func_name)
            assert(ida_typeinf.apply_cdecl(None, func_ea, func_proto))


    def dbg_run_to(self, pid, tid, ea):
        log("'run_to' reached its target location. Performing appcalls.")

        for func_name in ["ref4", "ref8"]:
            int_value = ida_idd.Appcall.int64(5)
            int_ptr = ida_idd.Appcall.byref(int_value)
            if ida_idd.Appcall[func_name](int_ptr):
                log("Appcall (%s) succeeded: int_value.value=%s, int_ptr.value=%s" % (
                    func_name,
                    int_value.value,
                    int_ptr.value))
            else:
                log("Appcall (%s) failed" % func_name)


    def run(self):
        log("Running program up to current address, and letting the hooks do the rest")
        assert(ida_dbg.run_to(ida_kernwin.get_screen_ea()))



"""
summary: showcasing `ida_bytes.bin_search`

description:
  IDAPython's ida_bytes.bin_search function is pretty powerful,
  but can be tough to figure out at first. This example introduces

   * `ida_bytes.bin_search`, and
   * `ida_bytes.parse_binpat_str`

  in order to implement a simple replacement for the
  'Search > Sequence of bytes...' dialog, that lets users
  search for sequences of bytes that compose string literals
  in the binary file (either in the default 1-byte-per-char
  encoding, or as UTF-16.)
"""

import ida_kernwin
import ida_bytes
import ida_ida
import ida_idaapi
import ida_nalt

class search_strlit_form_t(ida_kernwin.Form):
    def __init__(self):
        ida_kernwin.Form.__init__(
            self,
            r"""Please enter string literal

<Text: {Text}>
<#UTF16-BE if file is big-endian, UTF16-LE otherwise#As UTF-16: {UTF16}>{Encoding}>
""",
            {
                "Text" : ida_kernwin.Form.StringInput(),
                "Encoding" : ida_kernwin.Form.ChkGroupControl(("UTF16",)),
            })

class search_strlit_ah_t(ida_kernwin.action_handler_t):
    def __init__(self):
        ida_kernwin.action_handler_t.__init__(self)

    def activate(self, ctx):
        f = search_strlit_form_t()
        f, args = f.Compile()
        ok = f.Execute()
        if ok:
            current_ea = ida_kernwin.get_screen_ea()
            patterns = ida_bytes.compiled_binpat_vec_t()
            encoding = ida_nalt.get_default_encoding_idx(
                ida_nalt.BPU_2B if f.Encoding.value else ida_nalt.BPU_1B)
            # string literals must be quoted. That's how parse_binpat_str
            # recognizes them (we want to be careful though: the user
            # might type in something like 'L"hello"', which should
            # decode to the IDB-specific wide-char set of bytes)
            text = f.Text.value
            if text.find('"') < 0:
                text = '"%s"' % text
            err = ida_bytes.parse_binpat_str(
                patterns,
                current_ea,
                text,
                10, # radix (not that it matters though, since we're all about string literals)
                encoding)
            if not err:
                ea = ida_bytes.bin_search(
                    current_ea,
                    ida_ida.inf_get_max_ea(),
                    patterns,
                    ida_bytes.BIN_SEARCH_FORWARD
                  | ida_bytes.BIN_SEARCH_NOBREAK
                  | ida_bytes.BIN_SEARCH_NOSHOW)
                ok = ea != ida_idaapi.BADADDR
                if ok:
                    ida_kernwin.jumpto(ea)
            else:
                print("Failed parsing binary pattern: \"%s\"" % err)
        return ok

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET \
            if ctx.widget_type == ida_kernwin.BWN_DISASM \
            else ida_kernwin.AST_DISABLE_FOR_WIDGET


ACTION_NAME = "bin_search:search"
ACTION_SHORTCUT = "Ctrl+Shift+S"

if ida_kernwin.register_action(
        ida_kernwin.action_desc_t(
            ACTION_NAME,
            "Search for string literal",
            search_strlit_ah_t(),
            ACTION_SHORTCUT)):
    print("Please use \"%s\" to search for string literals" % ACTION_SHORTCUT)
